@@ -6,21 +6,23 @@
 /* SYNOPSIS
 /*	\fBanvil\fR [generic Postfix daemon options]
 /* DESCRIPTION
-/*	The Postfix \fBanvil\fR(8) server maintains short-term statistics
-/*	to defend against clients that hammer a server with either too
-/*	many simultaneous sessions, or with too many successive requests
-/*	within a configurable time interval.
-/*	This server is designed to run under control by the Postfix
-/*	\fBmaster\fR(8) server.
+/*	The Postfix \fBanvil\fR(8) server maintains statistics about
+/*	client connection counts or client request rates. This
+/*	information can be used to defend against clients that
+/*	hammer a server with either too many simultaneous sessions,
+/*	or with too many successive requests within a configurable
+/*	time interval.  This server is designed to run under control
+/*	by the Postfix \fBmaster\fR(8) server.
 /*
-/*	The \fBanvil\fR(8) server maintains no persistent database. Standard
-/*	library utilities do not meet Postfix performance and robustness
-/*	requirements.
-/* CONNECTION COUNT/RATE LIMITING
+/*	In the following text, \fBident\fR specifies a (service,
+/*	client) combination. The exact syntax of that information
+/*	is application-dependent; the \fBanvil\fR(8) server does
+/*	not care.
+/* CONNECTION COUNT/RATE CONTROL
 /* .ad
 /* .fi
-/*	When a remote client connects, a connection count (or rate) limited
-/*	server should send the following request to the \fBanvil\fR(8) server:
+/*	To register a new connection send the following request to
+/*	the \fBanvil\fR(8) server:
 /* .PP
 /* .in +4
 /*	\fBrequest=connect\fR
@@ -28,11 +30,10 @@
 /*	\fBident=\fIstring\fR
 /* .in
 /* .PP
-/*	This registers a new connection for the (service, client)
-/*	combination specified with \fBident\fR. The \fBanvil\fR(8) server
-/*	answers with the number of simultaneous connections and the
-/*	number of connections per unit time for that (service, client)
-/*	combination:
+/*	The \fBanvil\fR(8) server answers with the number of
+/*	simultaneous connections and the number of connections per
+/*	unit time for the (service, client) combination specified
+/*	with \fBident\fR:
 /* .PP
 /* .in +4
 /*	\fBstatus=0\fR
@@ -42,13 +43,8 @@
 /*	\fBrate=\fInumber\fR
 /* .in
 /* .PP
-/*	The \fBrate\fR is computed as the number of connections
-/*	that were registered in the current "time unit" interval.
-/*	It is left up to the server to decide if the remote client
-/*	exceeds the connection count (or rate) limit.
-/* .PP
-/*	When a remote client disconnects, a connection count (or rate) limited
-/*	server should send the following request to the \fBanvil\fR(8) server:
+/*	To register a disconnect event send the following request
+/*	to the \fBanvil\fR(8) server:
 /* .PP
 /* .in +4
 /*	\fBrequest=disconnect\fR
@@ -56,17 +52,14 @@
 /*	\fBident=\fIstring\fR
 /* .in
 /* .PP
-/*	This registers a disconnect event for the (service, client)
-/*	combination specified with \fBident\fR. The \fBanvil\fR(8)
-/*	server replies with:
+/*	The \fBanvil\fR(8) server replies with:
 /* .PP
 /* .ti +4
 /*	\fBstatus=0\fR
-/* MESSAGE RATE LIMITING
+/* MESSAGE RATE CONTROL
 /* .ad
 /* .fi
-/*	When a remote client sends a message delivery request, a
-/*	message rate limited server should send the following
+/*	To register a message delivery request send the following
 /*	request to the \fBanvil\fR(8) server:
 /* .PP
 /* .in +4
@@ -75,26 +68,20 @@
 /*	\fBident=\fIstring\fR
 /* .in
 /* .PP
-/*	This registers a message delivery request for the (service, client)
-/*	combination specified with \fBident\fR. The \fBanvil\fR(8) server
-/*	answers with the number of message delivery requests per unit time
-/*	for that (service, client) combination:
+/*	The \fBanvil\fR(8) server answers with the number of message
+/*	delivery requests per unit time for the (service, client)
+/*	combination specified with \fBident\fR:
 /* .PP
 /* .in +4
 /*	\fBstatus=0\fR
 /* .br
 /*	\fBrate=\fInumber\fR
 /* .in
-/* .PP
-/*	In order to prevent the \fBanvil\fR(8) server from discarding client
-/*	request rates too early or too late, a message rate limited
-/*	service should also register connect/disconnect events.
-/* RECIPIENT RATE LIMITING
+/* RECIPIENT RATE CONTROL
 /* .ad
 /* .fi
-/*	When a remote client sends a recipient address, a recipient
-/*	rate limited server should send the following request to
-/*	the \fBanvil\fR(8) server:
+/*	To register a recipient request send the following request
+/*	to the \fBanvil\fR(8) server:
 /* .PP
 /* .in +4
 /*	\fBrequest=recipient\fR
@@ -102,10 +89,33 @@
 /*	\fBident=\fIstring\fR
 /* .in
 /* .PP
-/*	This registers a recipient request for the (service, client)
-/*	combination specified with \fBident\fR. The \fBanvil\fR(8) server
-/*	answers with the number of recipient addresses per unit time
-/*	for that (service, client) combination:
+/*	The \fBanvil\fR(8) server answers with the number of recipient
+/*	addresses per unit time for the (service, client) combination
+/*	specified with \fBident\fR:
+/* .PP
+/* .in +4
+/*	\fBstatus=0\fR
+/* .br
+/*	\fBrate=\fInumber\fR
+/* .in
+/* TLS SESSION NEGOTIATION RATE CONTROL
+/* .ad
+/* .fi
+/*	The features described in this section are available with
+/*	Postfix 2.3 and later.
+/*
+/*	To register a request for a new (i.e. not cached) TLS session
+/*	send the following request to the \fBanvil\fR(8) server:
+/* .PP
+/* .in +4
+/*	\fBrequest=newtls\fR
+/* .br
+/*	\fBident=\fIstring\fR
+/* .in
+/* .PP
+/*	The \fBanvil\fR(8) server answers with the number of new
+/*	TLS session requests per unit time for the (service, client)
+/*	combination specified with \fBident\fR:
 /* .PP
 /* .in +4
 /*	\fBstatus=0\fR
@@ -113,21 +123,39 @@
 /*	\fBrate=\fInumber\fR
 /* .in
 /* .PP
-/*	In order to prevent the \fBanvil\fR(8) server from discarding client
-/*	request rates too early or too late, a recipient rate limited
-/*	service should also register connect/disconnect events.
+/*	To retrieve new TLS session request rate information without
+/*	updating the counter information, send:
+/* .PP
+/* .in +4
+/*	\fBrequest=newtls_report\fR
+/* .br
+/*	\fBident=\fIstring\fR
+/* .in
+/* .PP
+/*	The \fBanvil\fR(8) server answers with the number of new
+/*	TLS session requests per unit time for the (service, client)
+/*	combination specified with \fBident\fR:
+/* .PP
+/* .in +4
+/*	\fBstatus=0\fR
+/* .br
+/*	\fBrate=\fInumber\fR
+/* .in
 /* SECURITY
 /* .ad
 /* .fi
 /*	The \fBanvil\fR(8) server does not talk to the network or to local
 /*	users, and can run chrooted at fixed low privilege.
 /*
-/*	The \fBanvil\fR(8) server maintains an in-memory table with information
-/*	about recent clients of a connection count (or rate) limited service.
-/*	Although state is kept only temporarily, this may require a lot of
-/*	memory on systems that handle connections from many remote clients.
-/*	To reduce memory usage, reduce the time unit over which state
-/*	is kept.
+/*	The \fBanvil\fR(8) server maintains an in-memory table with
+/*	information about recent clients requests.  No persistent
+/*	state is kept because standard system library routines are
+/*	not sufficiently robust for update-intensive applications.
+/*
+/*	Although the in-memory state is kept only temporarily, this
+/*	may require a lot of memory on systems that handle connections
+/*	from many remote clients.  To reduce memory usage, reduce
+/*	the time unit over which state is kept.
 /* DIAGNOSTICS
 /*	Problems and transactions are logged to \fBsyslogd\fR(8).
 /*
@@ -147,12 +175,20 @@
 /*	can have only one remote client at a time. If a server reports
 /*	multiple simultaneous clients, all but the last reported client
 /*	are ignored.
+/*
+/*	The \fBanvil\fR(8) server automatically discards client
+/*	request information after it expires.  To prevent the
+/*	\fBanvil\fR(8) server from discarding client request rate
+/*	information too early or too late, a rate limited service
+/*	should always register connect/disconnect events even when
+/*	it does not explicitly limit them.
 /* CONFIGURATION PARAMETERS
 /* .ad
 /* .fi
-/*	Changes to \fBmain.cf\fR are picked up automatically as \fBanvil\fR(8)
-/*	processes run for only a limited amount of time. Use the command
-/*	"\fBpostfix reload\fR" to speed up a change.
+/*	On low-traffic mail systems, changes to \fBmain.cf\fR are
+/*	picked up automatically as \fBanvil\fR(8) processes run for
+/*	only a limited amount of time. On other mail systems, use
+/*	the command "\fBpostfix reload\fR" to speed up a change.
 /*
 /*	The text below provides only a parameter summary. See
 /*	\fBpostconf\fR(5) for more details including examples.
@@ -240,41 +276,17 @@
 
 /* Application-specific. */
 
+ /*
+  * Configuration parameters.
+  */
 int     var_anvil_time_unit;
 int     var_anvil_stat_time;
 
  /*
-  * State.
+  * Global dynamic state.
   */
 static HTABLE *anvil_remote_map;	/* indexed by service+ remote client */
 
- /*
-  * Absent a real-time query interface, these are logged at process exit time
-  * and at regular intervals.
-  */
-static int max_count;
-static char *max_count_user;
-static time_t max_count_time;
-
-static int max_rate;
-static char *max_rate_user;
-static time_t max_rate_time;
-
-static int max_mail;
-static char *max_mail_user;
-static time_t max_mail_time;
-
-static int max_rcpt;
-static char *max_rcpt_user;
-static time_t max_rcpt_time;
-
-static int max_newtls;
-static char *max_newtls_user;
-static time_t max_newtls_time;
-
-static int max_cache;
-static time_t max_cache_time;
-
  /*
   * Remote connection state, one instance for each (service, client) pair.
   */
@@ -284,24 +296,19 @@ typedef struct {
     int     rate;			/* connection rate */
     int     mail;			/* message rate */
     int     rcpt;			/* recipient rate */
-    int     newtls;			/* newtls rate */
+    int     ntls;			/* new TLS session rate */
     time_t  start;			/* time of first rate sample */
 } ANVIL_REMOTE;
 
  /*
-  * Local server state, one per server instance. This allows us to clean up
-  * connection state when a local server goes away without cleaning up.
+  * Local server state, one instance per anvil client connection. This allows
+  * us to clean up remote connection state when a local server goes away
+  * without cleaning up.
   */
 typedef struct {
     ANVIL_REMOTE *anvil_remote;		/* XXX should be list */
 } ANVIL_LOCAL;
 
- /*
-  * Silly little macros.
-  */
-#define STR(x)			vstring_str(x)
-#define STREQ(x,y)		(strcmp((x), (y)) == 0)
-
  /*
   * The following operations are implemented as macros with recognizable
   * names so that we don't lose sight of what the code is trying to do.
@@ -312,14 +319,14 @@ typedef struct {
 
 /* Create new (service, client) state. */
 
-#define ANVIL_REMOTE_FIRST(remote, id) \
+#define ANVIL_REMOTE_FIRST_CONN(remote, id) \
     do { \
 	(remote)->ident = mystrdup(id); \
 	(remote)->count = 1; \
 	(remote)->rate = 1; \
 	(remote)->mail = 0; \
 	(remote)->rcpt = 0; \
-	(remote)->newtls = 0; \
+	(remote)->ntls = 0; \
 	(remote)->start = event_time(); \
     } while(0)
 
@@ -331,66 +338,41 @@ typedef struct {
 	myfree((char *) (remote)); \
     } while(0)
 
-/* Add connection to (service, client) state. */
+/* Reset or update rate information for existing (service, client) state. */
 
-#define ANVIL_REMOTE_NEXT(remote) \
+#define ANVIL_REMOTE_RSET_RATE(remote, _start) \
+    do { \
+	(remote)->rate = 0; \
+	(remote)->mail = 0; \
+	(remote)->rcpt = 0; \
+	(remote)->ntls = 0; \
+	(remote)->start = _start; \
+    } while(0)
+
+#define ANVIL_REMOTE_INCR_RATE(remote, _what) \
     do { \
 	time_t _now = event_time(); \
-	if ((remote)->start + var_anvil_time_unit < _now) { \
-	    (remote)->rate = 1; \
-	    (remote)->mail = 0; \
-	    (remote)->rcpt = 0; \
-	    (remote)->newtls = 0; \
-	    (remote)->start = _now; \
-	} else if ((remote)->rate < INT_MAX) { \
-	    (remote)->rate += 1; \
-	} \
+	if ((remote)->start + var_anvil_time_unit < _now) \
+	    ANVIL_REMOTE_RSET_RATE((remote), _now); \
+	if ((remote)->_what < INT_MAX) \
+            (remote)->_what += 1; \
+    } while(0)
+
+/* Update existing (service, client) state. */
+
+#define ANVIL_REMOTE_NEXT_CONN(remote) \
+    do { \
+	ANVIL_REMOTE_INCR_RATE((remote), rate); \
 	if ((remote)->count == 0) \
 	    event_cancel_timer(anvil_remote_expire, (char *) remote); \
 	(remote)->count++; \
     } while(0)
 
-#define ANVIL_ADD_MAIL(remote) \
-    do { \
-	time_t _now = event_time(); \
-	if ((remote)->start + var_anvil_time_unit < _now) { \
-	    (remote)->rate = 0; \
-	    (remote)->mail = 1; \
-	    (remote)->rcpt = 0; \
-	    (remote)->newtls = 0; \
-	    (remote)->start = _now; \
-	} else if ((remote)->mail < INT_MAX) { \
-            (remote)->mail += 1; \
-	} \
-    } while(0)
+#define ANVIL_REMOTE_INCR_MAIL(remote)	ANVIL_REMOTE_INCR_RATE((remote), mail)
 
-#define ANVIL_ADD_RCPT(remote) \
-    do { \
-	time_t _now = event_time(); \
-	if ((remote)->start + var_anvil_time_unit < _now) { \
-	    (remote)->rate = 0; \
-	    (remote)->mail = 0; \
-	    (remote)->rcpt = 1; \
-	    (remote)->newtls = 0; \
-	    (remote)->start = _now; \
-	} else if ((remote)->rcpt < INT_MAX) { \
-            (remote)->rcpt += 1; \
-	} \
-    } while(0)
+#define ANVIL_REMOTE_INCR_RCPT(remote)	ANVIL_REMOTE_INCR_RATE((remote), rcpt)
 
-#define ANVIL_ADD_STARTTLS(remote) \
-    do { \
-	time_t _now = event_time(); \
-	if ((remote)->start + var_anvil_time_unit < _now) { \
-	    (remote)->rate = 0; \
-	    (remote)->mail = 0; \
-	    (remote)->rcpt = 0; \
-	    (remote)->newtls = 1; \
-	    (remote)->start = _now; \
-	} else if ((remote)->rcpt < INT_MAX) { \
-            (remote)->newtls += 1; \
-	} \
-    } while(0)
+#define ANVIL_REMOTE_INCR_NTLS(remote)	ANVIL_REMOTE_INCR_RATE((remote), ntls)
 
 /* Drop connection from (service, client) state. */
 
@@ -410,7 +392,7 @@ typedef struct {
 	(local)->anvil_remote = 0; \
     } while(0)
 
-/* Add connection to local server. */
+/* Add remote connection to local server. */
 
 #define ANVIL_LOCAL_ADD_ONE(local, remote) \
     do { \
@@ -420,12 +402,12 @@ typedef struct {
 	(local)->anvil_remote = (remote); \
     } while(0)
 
-/* Test if this remote site is listed for this local client. */
+/* Test if this remote connection is listed for this local server. */
 
 #define ANVIL_LOCAL_REMOTE_LINKED(local, remote) \
     ((local)->anvil_remote == (remote))
 
-/* Drop connection from local server. */
+/* Drop specific remote connection from local server. */
 
 #define ANVIL_LOCAL_DROP_ONE(local, remote) \
     do { \
@@ -434,7 +416,7 @@ typedef struct {
 	    (local)->anvil_remote = 0; \
     } while(0)
 
-/* Drop all connections from local server. */
+/* Drop all remote connections from local server. */
 
 #define ANVIL_LOCAL_DROP_ALL(stream, local) \
     do { \
@@ -443,12 +425,80 @@ typedef struct {
 	    anvil_remote_disconnect((stream), (local)->anvil_remote->ident); \
     } while (0)
 
+ /*
+  * Lookup table to map request names to action routines.
+  */
+typedef struct {
+    const char *name;
+    void    (*action) (VSTREAM *, const char *);
+} ANVIL_REQ_TABLE;
+
+ /*
+  * Run-time statistics for maximal connection counts and event rates. These
+  * store the peak resource usage, remote connection, and time. Absent a
+  * query interface, this information is logged at process exit time and at
+  * configurable intervals.
+  */
+typedef struct {
+    int     value;			/* peak value */
+    char   *ident;			/* lookup key */
+    time_t  when;			/* time of peak value */
+} ANVIL_MAX;
+
+static ANVIL_MAX max_conn_count;	/* peak connection count */
+static ANVIL_MAX max_conn_rate;		/* peak connection rate */
+static ANVIL_MAX max_mail_rate;		/* peak message rate */
+static ANVIL_MAX max_rcpt_rate;		/* peak recipient rate */
+static ANVIL_MAX max_ntls_rate;		/* peak new TLS session rate */
+
+static int max_cache_size;		/* peak cache size */
+static time_t max_cache_time;		/* time of peak size */
+
+/* Update/report peak usage. */
+
+#define ANVIL_MAX_UPDATE(_max, _value, _ident) \
+    do { \
+	_max.value = _value; \
+	if (_max.ident == 0) { \
+	    _max.ident = mystrdup(_ident); \
+	} else if (!STREQ(_max.ident, _ident)) { \
+	    myfree(_max.ident); \
+	    _max.ident = mystrdup(_ident); \
+	} \
+	_max.when = event_time(); \
+    } while (0)
+
+#define ANVIL_MAX_RATE_REPORT(_max, _name) \
+    do { \
+	if (_max.value > 0) { \
+	    msg_info("statistics: max " _name " rate %d/%ds for (%s) at %.15s", \
+		_max.value, var_anvil_time_unit, \
+		_max.ident, ctime(&_max.when) + 4); \
+	    _max.value = 0; \
+	} \
+    } while (0);
+
+#define ANVIL_MAX_COUNT_REPORT(_max, _name) \
+    do { \
+	if (_max.value > 0) { \
+	    msg_info("statistics: max " _name " count %d for (%s) at %.15s", \
+		_max.value, _max.ident, ctime(&_max.when) + 4); \
+	    _max.value = 0; \
+	} \
+    } while (0);
+
+ /*
+  * Silly little macros.
+  */
+#define STR(x)			vstring_str(x)
+#define STREQ(x,y)		(strcmp((x), (y)) == 0)
+
 /* anvil_remote_expire - purge expired connection state */
 
 static void anvil_remote_expire(int unused_event, char *context)
 {
     ANVIL_REMOTE *anvil_remote = (ANVIL_REMOTE *) context;
-    char   *myname = "anvil_remote_expire";
+    const char *myname = "anvil_remote_expire";
 
     if (msg_verbose)
 	msg_info("%s %s", myname, anvil_remote->ident);
@@ -471,9 +521,7 @@ static void anvil_remote_expire(int unused_event, char *context)
 static void anvil_remote_lookup(VSTREAM *client_stream, const char *ident)
 {
     ANVIL_REMOTE *anvil_remote;
-    char   *myname = "anvil_remote_lookup";
-    HTABLE_INFO **ht_info;
-    HTABLE_INFO **ht;
+    const char *myname = "anvil_remote_lookup";
 
     if (msg_verbose)
 	msg_info("%s fd=%d stream=0x%lx ident=%s",
@@ -483,39 +531,31 @@ static void anvil_remote_lookup(VSTREAM *client_stream, const char *ident)
     /*
      * Look up remote client information.
      */
-    if (STREQ(ident, "*")) {
-	attr_print_plain(client_stream, ATTR_FLAG_MORE,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
-			 ATTR_TYPE_END);
-	ht_info = htable_list(anvil_remote_map);
-	for (ht = ht_info; *ht; ht++) {
-	    anvil_remote = (ANVIL_REMOTE *) ht[0]->value;
-	    attr_print_plain(client_stream, ATTR_FLAG_MORE,
-			     ATTR_TYPE_STR, ANVIL_ATTR_IDENT, ht[0]->key,
-		       ATTR_TYPE_NUM, ANVIL_ATTR_COUNT, anvil_remote->count,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_RATE, anvil_remote->rate,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_MAIL, anvil_remote->mail,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_RCPT, anvil_remote->rcpt,
-			     ATTR_TYPE_END);
-	}
-	attr_print_plain(client_stream, ATTR_FLAG_NONE, ATTR_TYPE_END);
-	myfree((char *) ht_info);
-    } else if ((anvil_remote =
-	      (ANVIL_REMOTE *) htable_find(anvil_remote_map, ident)) == 0) {
+    if ((anvil_remote =
+	 (ANVIL_REMOTE *) htable_find(anvil_remote_map, ident)) == 0) {
 	attr_print_plain(client_stream, ATTR_FLAG_NONE,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_STATUS, ANVIL_STAT_FAIL,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_COUNT, 0,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_RATE, 0,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_MAIL, 0,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_RCPT, 0,
+			 ATTR_TYPE_INT, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
+			 ATTR_TYPE_INT, ANVIL_ATTR_COUNT, 0,
+			 ATTR_TYPE_INT, ANVIL_ATTR_RATE, 0,
+			 ATTR_TYPE_INT, ANVIL_ATTR_MAIL, 0,
+			 ATTR_TYPE_INT, ANVIL_ATTR_RCPT, 0,
+			 ATTR_TYPE_INT, ANVIL_ATTR_NTLS, 0,
 			 ATTR_TYPE_END);
     } else {
+
+	/*
+	 * Do not report stale information.
+	 */
+	if (anvil_remote->start != 0
+	    && anvil_remote->start + var_anvil_time_unit < event_time())
+	    ANVIL_REMOTE_RSET_RATE(anvil_remote, 0);
 	attr_print_plain(client_stream, ATTR_FLAG_NONE,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
-		       ATTR_TYPE_NUM, ANVIL_ATTR_COUNT, anvil_remote->count,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_RATE, anvil_remote->rate,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_MAIL, anvil_remote->mail,
-			 ATTR_TYPE_NUM, ANVIL_ATTR_RCPT, anvil_remote->rcpt,
+			 ATTR_TYPE_INT, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
+		       ATTR_TYPE_INT, ANVIL_ATTR_COUNT, anvil_remote->count,
+			 ATTR_TYPE_INT, ANVIL_ATTR_RATE, anvil_remote->rate,
+			 ATTR_TYPE_INT, ANVIL_ATTR_MAIL, anvil_remote->mail,
+			 ATTR_TYPE_INT, ANVIL_ATTR_RCPT, anvil_remote->rcpt,
+			 ATTR_TYPE_INT, ANVIL_ATTR_NTLS, anvil_remote->ntls,
 			 ATTR_TYPE_END);
     }
 }
@@ -526,7 +566,7 @@ static ANVIL_REMOTE *anvil_remote_conn_update(VSTREAM *client_stream, const char
 {
     ANVIL_REMOTE *anvil_remote;
     ANVIL_LOCAL *anvil_local;
-    char   *myname = "anvil_remote_conn_update";
+    const char *myname = "anvil_remote_conn_update";
 
     if (msg_verbose)
 	msg_info("%s fd=%d stream=0x%lx ident=%s",
@@ -542,19 +582,19 @@ static ANVIL_REMOTE *anvil_remote_conn_update(VSTREAM *client_stream, const char
     if ((anvil_remote =
 	 (ANVIL_REMOTE *) htable_find(anvil_remote_map, ident)) == 0) {
 	anvil_remote = (ANVIL_REMOTE *) mymalloc(sizeof(*anvil_remote));
-	ANVIL_REMOTE_FIRST(anvil_remote, ident);
+	ANVIL_REMOTE_FIRST_CONN(anvil_remote, ident);
 	htable_enter(anvil_remote_map, ident, (char *) anvil_remote);
-	if (max_cache < anvil_remote_map->used) {
-	    max_cache = anvil_remote_map->used;
+	if (max_cache_size < anvil_remote_map->used) {
+	    max_cache_size = anvil_remote_map->used;
 	    max_cache_time = event_time();
 	}
     } else {
-	ANVIL_REMOTE_NEXT(anvil_remote);
+	ANVIL_REMOTE_NEXT_CONN(anvil_remote);
     }
 
     /*
-     * Record this connection under the local client information, so that we
-     * can clean up all its connection state when the local client goes away.
+     * Record this connection under the local server information, so that we
+     * can clean up all its connection state when the local server goes away.
      */
     if ((anvil_local = (ANVIL_LOCAL *) vstream_context(client_stream)) == 0) {
 	anvil_local = (ANVIL_LOCAL *) mymalloc(sizeof(*anvil_local));
@@ -583,37 +623,21 @@ static void anvil_remote_connect(VSTREAM *client_stream, const char *ident)
     anvil_remote = anvil_remote_conn_update(client_stream, ident);
 
     /*
-     * Respond to the local client.
+     * Respond to the local server.
      */
     attr_print_plain(client_stream, ATTR_FLAG_NONE,
-		     ATTR_TYPE_NUM, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
-		     ATTR_TYPE_NUM, ANVIL_ATTR_COUNT, anvil_remote->count,
-		     ATTR_TYPE_NUM, ANVIL_ATTR_RATE, anvil_remote->rate,
+		     ATTR_TYPE_INT, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
+		     ATTR_TYPE_INT, ANVIL_ATTR_COUNT, anvil_remote->count,
+		     ATTR_TYPE_INT, ANVIL_ATTR_RATE, anvil_remote->rate,
 		     ATTR_TYPE_END);
 
     /*
-     * Update local statistics.
+     * Update peak statistics.
      */
-    if (anvil_remote->rate > max_rate) {
-	max_rate = anvil_remote->rate;
-	if (max_rate_user == 0) {
-	    max_rate_user = mystrdup(anvil_remote->ident);
-	} else if (!STREQ(max_rate_user, anvil_remote->ident)) {
-	    myfree(max_rate_user);
-	    max_rate_user = mystrdup(anvil_remote->ident);
-	}
-	max_rate_time = event_time();
-    }
-    if (anvil_remote->count > max_count) {
-	max_count = anvil_remote->count;
-	if (max_count_user == 0) {
-	    max_count_user = mystrdup(anvil_remote->ident);
-	} else if (!STREQ(max_count_user, anvil_remote->ident)) {
-	    myfree(max_count_user);
-	    max_count_user = mystrdup(anvil_remote->ident);
-	}
-	max_count_time = event_time();
-    }
+    if (anvil_remote->rate > max_conn_rate.value)
+	ANVIL_MAX_UPDATE(max_conn_rate, anvil_remote->rate, anvil_remote->ident);
+    if (anvil_remote->count > max_conn_count.value)
+	ANVIL_MAX_UPDATE(max_conn_count, anvil_remote->count, anvil_remote->ident);
 }
 
 /* anvil_remote_mail - register message delivery request */
@@ -630,27 +654,19 @@ static void anvil_remote_mail(VSTREAM *client_stream, const char *ident)
 	anvil_remote = anvil_remote_conn_update(client_stream, ident);
 
     /*
-     * Update message delivery request rate and respond to local client.
+     * Update message delivery request rate and respond to local server.
      */
-    ANVIL_ADD_MAIL(anvil_remote);
+    ANVIL_REMOTE_INCR_MAIL(anvil_remote);
     attr_print_plain(client_stream, ATTR_FLAG_NONE,
-		     ATTR_TYPE_NUM, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
-		     ATTR_TYPE_NUM, ANVIL_ATTR_RATE, anvil_remote->mail,
+		     ATTR_TYPE_INT, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
+		     ATTR_TYPE_INT, ANVIL_ATTR_RATE, anvil_remote->mail,
 		     ATTR_TYPE_END);
 
     /*
-     * Update local statistics.
+     * Update peak statistics.
      */
-    if (anvil_remote->mail > max_mail) {
-	max_mail = anvil_remote->mail;
-	if (max_mail_user == 0) {
-	    max_mail_user = mystrdup(anvil_remote->ident);
-	} else if (!STREQ(max_mail_user, anvil_remote->ident)) {
-	    myfree(max_mail_user);
-	    max_mail_user = mystrdup(anvil_remote->ident);
-	}
-	max_mail_time = event_time();
-    }
+    if (anvil_remote->mail > max_mail_rate.value)
+	ANVIL_MAX_UPDATE(max_mail_rate, anvil_remote->mail, anvil_remote->ident);
 }
 
 /* anvil_remote_rcpt - register recipient address event */
@@ -667,27 +683,19 @@ static void anvil_remote_rcpt(VSTREAM *client_stream, const char *ident)
 	anvil_remote = anvil_remote_conn_update(client_stream, ident);
 
     /*
-     * Update recipient address rate and respond to local client.
+     * Update recipient address rate and respond to local server.
      */
-    ANVIL_ADD_RCPT(anvil_remote);
+    ANVIL_REMOTE_INCR_RCPT(anvil_remote);
     attr_print_plain(client_stream, ATTR_FLAG_NONE,
-		     ATTR_TYPE_NUM, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
-		     ATTR_TYPE_NUM, ANVIL_ATTR_RATE, anvil_remote->rcpt,
+		     ATTR_TYPE_INT, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
+		     ATTR_TYPE_INT, ANVIL_ATTR_RATE, anvil_remote->rcpt,
 		     ATTR_TYPE_END);
 
     /*
-     * Update local statistics.
+     * Update peak statistics.
      */
-    if (anvil_remote->rcpt > max_rcpt) {
-	max_rcpt = anvil_remote->rcpt;
-	if (max_rcpt_user == 0) {
-	    max_rcpt_user = mystrdup(anvil_remote->ident);
-	} else if (!STREQ(max_rcpt_user, anvil_remote->ident)) {
-	    myfree(max_rcpt_user);
-	    max_rcpt_user = mystrdup(anvil_remote->ident);
-	}
-	max_rcpt_time = event_time();
-    }
+    if (anvil_remote->rcpt > max_rcpt_rate.value)
+	ANVIL_MAX_UPDATE(max_rcpt_rate, anvil_remote->rcpt, anvil_remote->ident);
 }
 
 /* anvil_remote_newtls - register newtls event */
@@ -704,27 +712,53 @@ static void anvil_remote_newtls(VSTREAM *client_stream, const char *ident)
 	anvil_remote = anvil_remote_conn_update(client_stream, ident);
 
     /*
-     * Update newtls rate and respond to local client.
+     * Update newtls rate and respond to local server.
      */
-    ANVIL_ADD_STARTTLS(anvil_remote);
+    ANVIL_REMOTE_INCR_NTLS(anvil_remote);
     attr_print_plain(client_stream, ATTR_FLAG_NONE,
-		     ATTR_TYPE_NUM, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
-		     ATTR_TYPE_NUM, ANVIL_ATTR_RATE, anvil_remote->newtls,
+		     ATTR_TYPE_INT, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
+		     ATTR_TYPE_INT, ANVIL_ATTR_RATE, anvil_remote->ntls,
 		     ATTR_TYPE_END);
 
     /*
-     * Update local statistics.
+     * Update peak statistics.
      */
-    if (anvil_remote->newtls > max_newtls) {
-	max_newtls = anvil_remote->newtls;
-	if (max_newtls_user == 0) {
-	    max_newtls_user = mystrdup(anvil_remote->ident);
-	} else if (!STREQ(max_newtls_user, anvil_remote->ident)) {
-	    myfree(max_newtls_user);
-	    max_newtls_user = mystrdup(anvil_remote->ident);
-	}
-	max_newtls_time = event_time();
+    if (anvil_remote->ntls > max_ntls_rate.value)
+	ANVIL_MAX_UPDATE(max_ntls_rate, anvil_remote->ntls, anvil_remote->ident);
+}
+
+/* anvil_remote_newtls_stat - report newtls stats */
+
+static void anvil_remote_newtls_stat(VSTREAM *client_stream, const char *ident)
+{
+    ANVIL_REMOTE *anvil_remote;
+    int     rate;
+
+    /*
+     * Be prepared for "postfix reload" after "connect".
+     */
+    if ((anvil_remote =
+	 (ANVIL_REMOTE *) htable_find(anvil_remote_map, ident)) == 0) {
+	rate = 0;
     }
+
+    /*
+     * Do not report stale information.
+     */
+    else {
+	if (anvil_remote->start != 0
+	    && anvil_remote->start + var_anvil_time_unit < event_time())
+	    ANVIL_REMOTE_RSET_RATE(anvil_remote, 0);
+	rate = anvil_remote->ntls;
+    }
+
+    /*
+     * Respond to local server.
+     */
+    attr_print_plain(client_stream, ATTR_FLAG_NONE,
+		     ATTR_TYPE_INT, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
+		     ATTR_TYPE_INT, ANVIL_ATTR_RATE, rate,
+		     ATTR_TYPE_END);
 }
 
 /* anvil_remote_disconnect - report disconnect event */
@@ -733,7 +767,7 @@ static void anvil_remote_disconnect(VSTREAM *client_stream, const char *ident)
 {
     ANVIL_REMOTE *anvil_remote;
     ANVIL_LOCAL *anvil_local;
-    char   *myname = "anvil_remote_disconnect";
+    const char *myname = "anvil_remote_disconnect";
 
     if (msg_verbose)
 	msg_info("%s fd=%d stream=0x%lx ident=%s",
@@ -741,8 +775,8 @@ static void anvil_remote_disconnect(VSTREAM *client_stream, const char *ident)
 		 (unsigned long) client_stream, ident);
 
     /*
-     * Update local and remote info if this remote site is listed for this
-     * local client.
+     * Update local and remote info if this remote connection is listed for
+     * this local server.
      */
     if ((anvil_local = (ANVIL_LOCAL *) vstream_context(client_stream)) != 0
 	&& (anvil_remote =
@@ -756,10 +790,10 @@ static void anvil_remote_disconnect(VSTREAM *client_stream, const char *ident)
 		 myname, (unsigned long) anvil_local);
 
     /*
-     * Respond to the local client.
+     * Respond to the local server.
      */
     attr_print_plain(client_stream, ATTR_FLAG_NONE,
-		     ATTR_TYPE_NUM, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
+		     ATTR_TYPE_INT, ANVIL_ATTR_STATUS, ANVIL_STAT_OK,
 		     ATTR_TYPE_END);
 }
 
@@ -769,7 +803,7 @@ static void anvil_service_done(VSTREAM *client_stream, char *unused_service,
 			               char **unused_argv)
 {
     ANVIL_LOCAL *anvil_local;
-    char   *myname = "anvil_service_done";
+    const char *myname = "anvil_service_done";
 
     if (msg_verbose)
 	msg_info("%s fd=%d stream=0x%lx",
@@ -777,8 +811,8 @@ static void anvil_service_done(VSTREAM *client_stream, char *unused_service,
 		 (unsigned long) client_stream);
 
     /*
-     * Look up the local client, and get rid of open remote connection state
-     * that we still have for this local client. Do not destroy remote client
+     * Look up the local server, and get rid of any remote connection state
+     * that we still have for this local server. Do not destroy remote client
      * status information before it expires.
      */
     if ((anvil_local = (ANVIL_LOCAL *) vstream_context(client_stream)) != 0) {
@@ -792,12 +826,48 @@ static void anvil_service_done(VSTREAM *client_stream, char *unused_service,
 		 vstream_fileno(client_stream));
 }
 
+/* anvil_status_dump - log and reset extreme usage */
+
+static void anvil_status_dump(char *unused_name, char **unused_argv)
+{
+    ANVIL_MAX_RATE_REPORT(max_conn_rate, "connection");
+    ANVIL_MAX_COUNT_REPORT(max_conn_count, "connection");
+    ANVIL_MAX_RATE_REPORT(max_mail_rate, "message");
+    ANVIL_MAX_RATE_REPORT(max_rcpt_rate, "recipient");
+    ANVIL_MAX_RATE_REPORT(max_ntls_rate, "newtls");
+
+    if (max_cache_size > 0) {
+	msg_info("statistics: max cache size %d at %.15s",
+		 max_cache_size, ctime(&max_cache_time) + 4);
+	max_cache_size = 0;
+    }
+}
+
+/* anvil_status_update - log and reset extreme usage periodically */
+
+static void anvil_status_update(int unused_event, char *context)
+{
+    anvil_status_dump((char *) 0, (char **) 0);
+    event_request_timer(anvil_status_update, context, var_anvil_stat_time);
+}
+
 /* anvil_service - perform service for client */
 
 static void anvil_service(VSTREAM *client_stream, char *unused_service, char **argv)
 {
-    VSTRING *request = vstring_alloc(10);
-    VSTRING *ident = vstring_alloc(10);
+    static VSTRING *request;
+    static VSTRING *ident;
+    static ANVIL_REQ_TABLE request_table[] = {
+	ANVIL_REQ_CONN, anvil_remote_connect,
+	ANVIL_REQ_MAIL, anvil_remote_mail,
+	ANVIL_REQ_RCPT, anvil_remote_rcpt,
+	ANVIL_REQ_NTLS, anvil_remote_newtls,
+	ANVIL_REQ_DISC, anvil_remote_disconnect,
+	ANVIL_REQ_NTLS_STAT, anvil_remote_newtls_stat,
+	ANVIL_REQ_LOOKUP, anvil_remote_lookup,
+	0, 0,
+    };
+    ANVIL_REQ_TABLE *rp;
 
     /*
      * Sanity check. This service takes no command-line arguments.
@@ -805,6 +875,14 @@ static void anvil_service(VSTREAM *client_stream, char *unused_service, char **a
     if (argv[0])
 	msg_fatal("unexpected command-line argument: %s", argv[0]);
 
+    /*
+     * Initialize.
+     */
+    if (request == 0) {
+	request = vstring_alloc(10);
+	ident = vstring_alloc(10);
+    }
+
     /*
      * This routine runs whenever a client connects to the socket dedicated
      * to the client connection rate management service. All
@@ -818,23 +896,18 @@ static void anvil_service(VSTREAM *client_stream, char *unused_service, char **a
 			ATTR_TYPE_STR, ANVIL_ATTR_REQ, request,
 			ATTR_TYPE_STR, ANVIL_ATTR_IDENT, ident,
 			ATTR_TYPE_END) == 2) {
-	if (STREQ(STR(request), ANVIL_REQ_CONN)) {
-	    anvil_remote_connect(client_stream, STR(ident));
-	} else if (STREQ(STR(request), ANVIL_REQ_MAIL)) {
-	    anvil_remote_mail(client_stream, STR(ident));
-	} else if (STREQ(STR(request), ANVIL_REQ_RCPT)) {
-	    anvil_remote_rcpt(client_stream, STR(ident));
-	} else if (STREQ(STR(request), ANVIL_REQ_NEWTLS)) {
-	    anvil_remote_newtls(client_stream, STR(ident));
-	} else if (STREQ(STR(request), ANVIL_REQ_DISC)) {
-	    anvil_remote_disconnect(client_stream, STR(ident));
-	} else if (STREQ(STR(request), ANVIL_REQ_LOOKUP)) {
-	    anvil_remote_lookup(client_stream, STR(ident));
-	} else {
-	    msg_warn("unrecognized request: \"%s\", ignored", STR(request));
-	    attr_print_plain(client_stream, ATTR_FLAG_NONE,
-			  ATTR_TYPE_NUM, ANVIL_ATTR_STATUS, ANVIL_STAT_FAIL,
-			     ATTR_TYPE_END);
+	for (rp = request_table; /* see below */ ; rp++) {
+	    if (rp->name == 0) {
+		msg_warn("unrecognized request: \"%s\", ignored", STR(request));
+		attr_print_plain(client_stream, ATTR_FLAG_NONE,
+			  ATTR_TYPE_INT, ANVIL_ATTR_STATUS, ANVIL_STAT_FAIL,
+				 ATTR_TYPE_END);
+		break;
+	    }
+	    if (STREQ(rp->name, STR(request))) {
+		rp->action(client_stream, STR(ident));
+		break;
+	    }
 	}
 	vstream_fflush(client_stream);
     } else {
@@ -843,14 +916,10 @@ static void anvil_service(VSTREAM *client_stream, char *unused_service, char **a
     }
     if (msg_verbose)
 	msg_info("--- end request ---");
-    vstring_free(ident);
-    vstring_free(request);
 }
 
 /* post_jail_init - post-jail initialization */
 
-static void anvil_status_update(int, char *);
-
 static void post_jail_init(char *unused_name, char **unused_argv)
 {
 
@@ -868,54 +937,12 @@ static void post_jail_init(char *unused_name, char **unused_argv)
      * Do not limit the number of client requests.
      */
     var_use_limit = 0;
-}
 
-/* anvil_status_dump - log and reset extreme usage */
-
-static void anvil_status_dump(char *unused_name, char **unused_argv)
-{
-    if (max_rate > 0) {
-	msg_info("statistics: max connection rate %d/%ds for (%s) at %.15s",
-		 max_rate, var_anvil_time_unit,
-		 max_rate_user, ctime(&max_rate_time) + 4);
-	max_rate = 0;
-    }
-    if (max_count > 0) {
-	msg_info("statistics: max connection count %d for (%s) at %.15s",
-		 max_count, max_count_user, ctime(&max_count_time) + 4);
-	max_count = 0;
-    }
-    if (max_mail > 0) {
-	msg_info("statistics: max message rate %d/%ds for (%s) at %.15s",
-		 max_mail, var_anvil_time_unit,
-		 max_mail_user, ctime(&max_mail_time) + 4);
-	max_mail = 0;
-    }
-    if (max_rcpt > 0) {
-	msg_info("statistics: max recipient rate %d/%ds for (%s) at %.15s",
-		 max_rcpt, var_anvil_time_unit,
-		 max_rcpt_user, ctime(&max_rcpt_time) + 4);
-	max_rcpt = 0;
-    }
-    if (max_newtls > 0) {
-	msg_info("statistics: max newtls rate %d/%ds for (%s) at %.15s",
-		 max_newtls, var_anvil_time_unit,
-		 max_newtls_user, ctime(&max_newtls_time) + 4);
-	max_newtls = 0;
-    }
-    if (max_cache > 0) {
-	msg_info("statistics: max cache size %d at %.15s",
-		 max_cache, ctime(&max_cache_time) + 4);
-	max_cache = 0;
-    }
-}
-
-/* anvil_status_update - log and reset extreme usage periodically */
-
-static void anvil_status_update(int unused_event, char *context)
-{
-    anvil_status_dump((char *) 0, (char **) 0);
-    event_request_timer(anvil_status_update, context, var_anvil_stat_time);
+    /*
+     * Don't exit before the sampling interval ends.
+     */
+    if (var_idle_limit < var_anvil_time_unit)
+	var_idle_limit = var_anvil_time_unit;
 }
 
 /* main - pass control to the multi-threaded skeleton */
