@@ -46,6 +46,8 @@
 /*	RFC 822 (ARPA Internet Text Messages)
 /*	RFC 2045 (MIME: Format of Internet Message Bodies)
 /*	RFC 2046 (MIME: Media Types)
+/*	RFC 3463 (Enhanced Status Codes)
+/*	RFC 3464 (Delivery status notifications)
 /* DIAGNOSTICS
 /*	Problems and transactions are logged to \fBsyslogd\fR(8).
 /* BUGS
@@ -68,7 +70,13 @@
 /*	Message header that the Postfix \fBcleanup\fR(8) server inserts when a
 /*	message contains no To: or Cc: message header.
 /* .PP
-/*	Available in Postfix version 2.1 and later:
+/*	Available in Postfix version 2.1 only:
+/* .IP "\fBenable_errors_to (no)\fR"
+/*	Report mail delivery errors to the address specified with the
+/*	non-standard Errors-To: message header, instead of the envelope
+/*	sender address (this feature is removed with Postfix version 2.2, is
+/*	turned off by default with Postfix version 2.1, and is always turned on
+/*	with older Postfix versions).
 /* BUILT-IN CONTENT FILTERING CONTROLS
 /* .ad
 /* .fi
@@ -92,6 +100,65 @@
 /*	Optional lookup tables for content inspection of non-MIME message
 /*	headers in attached messages, as described in the \fBheader_checks\fR(5)
 /*	manual page.
+/* .PP
+/*	Available in Postfix version 2.3 and later:
+/* .IP "\fBmessage_reject_characters (empty)\fR"
+/*	The set of characters that Postfix will reject in message
+/*	content.
+/* .IP "\fBmessage_strip_characters (empty)\fR"
+/*	The set of characters that Postfix will remove from message
+/*	content.
+/* BEFORE QUEUE MILTER CONTROLS
+/* .ad
+/* .fi
+/*	As of version 2.3, Postfix supports the Sendmail version 8
+/*	Milter (mail filter) protocol. When mail is not received via
+/*	the smtpd(8) server, the cleanup(8) server will simulate
+/*	SMTP events to the extent that this is possible. For details
+/*	see the MILTER_README document.
+/* .IP "\fBnon_smtpd_milters (empty)\fR"
+/*	A list of Milter (mail filter) applications for new mail that
+/*	does not arrive via the Postfix \fBsmtpd\fR(8) server.
+/* .IP "\fBmilter_protocol (2)\fR"
+/*	The mail filter protocol version and optional protocol extensions
+/*	for communication with a Milter (mail filter) application.
+/* .IP "\fBmilter_default_action (tempfail)\fR"
+/*	The default action when a Milter (mail filter) application is
+/*	unavailable or mis-configured.
+/* .IP "\fBmilter_macro_daemon_name ($myhostname)\fR"
+/*	The {daemon_name} macro value for Milter (mail filter) applications.
+/* .IP "\fBmilter_macro_v ($mail_name $mail_version)\fR"
+/*	The {v} macro value for Milter (mail filter) applications.
+/* .IP "\fBmilter_connect_timeout (30s)\fR"
+/*	The time limit for connecting to a Milter (mail filter)
+/*	application, and for negotiating protocol options.
+/* .IP "\fBmilter_command_timeout (30s)\fR"
+/*	The time limit for sending an SMTP command to a Milter (mail
+/*	filter) application, and for receiving the response.
+/* .IP "\fBmilter_content_timeout (300s)\fR"
+/*	The time limit for sending message content to a Milter (mail
+/*	filter) application, and for receiving the response.
+/* .IP "\fBmilter_connect_macros (see postconf -n output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after completion of an SMTP connection.
+/* .IP "\fBmilter_helo_macros (see postconf -n output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the SMTP HELO or EHLO command.
+/* .IP "\fBmilter_mail_macros (see postconf -n output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the SMTP MAIL FROM command.
+/* .IP "\fBmilter_rcpt_macros (see postconf -n output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the SMTP RCPT TO command.
+/* .IP "\fBmilter_data_macros (see postconf -n output)\fR"
+/*	The macros that are sent to version 4 or higher Milter (mail
+/*	filter) applications after the SMTP DATA command.
+/* .IP "\fBmilter_unknown_command_macros (see postconf -n output)\fR"
+/*	The macros that are sent to version 3 or higher Milter (mail
+/*	filter) applications after an unknown SMTP command.
+/* .IP "\fBmilter_end_of_data_macros (see postconf -n output)\fR"
+/*	The macros that are sent to Milter (mail filter) applications
+/*	after the message end-of-data.
 /* MIME PROCESSING CONTROLS
 /* .ad
 /* .fi
@@ -179,8 +246,8 @@
 /*	mapping.
 /* .IP "\fBremote_header_rewrite_domain (empty)\fR"
 /*	Don't rewrite message headers from remote clients at all when
-/*	this parameter is empty; otherwise, rewrite remote message headers
-/*	and append the specified domain name to incomplete addresses.
+/*	this parameter is empty; otherwise, rewrite message headers and
+/*	append the specified domain name to incomplete addresses.
 /* RESOURCE AND RATE CONTROLS
 /* .ad
 /* .fi
@@ -226,6 +293,9 @@
 /* .IP "\fBdaemon_timeout (18000s)\fR"
 /*	How much time a Postfix daemon process may take to handle a
 /*	request before it is terminated by a built-in watchdog timer.
+/* .IP "\fBdelay_logging_resolution_limit (2)\fR"
+/*	The maximal number of digits after the decimal point when logging
+/*	sub-second delay values.
 /* .IP "\fBdelay_warning_time (0h)\fR"
 /*	The time after which the sender receives the message headers of
 /*	mail that is still queued.
@@ -283,6 +353,7 @@
 /* .na
 /* .nf
 /*	ADDRESS_REWRITING_README Postfix address manipulation
+/*	CONTENT_INSPECTION_README content inspection
 /* LICENSE
 /* .ad
 /* .fi
@@ -332,6 +403,7 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
     CLEANUP_STATE *state;
     int     flags;
     int     type = 0;
+    int     status;
 
     /*
      * Sanity check. This service takes no command-line arguments.
@@ -342,7 +414,7 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
     /*
      * Open a queue file and initialize state.
      */
-    state = cleanup_open();
+    state = cleanup_open(src);
 
     /*
      * Send the queue id to the client. Read client processing options. If we
@@ -353,7 +425,7 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
 	       ATTR_TYPE_STR, MAIL_ATTR_QUEUEID, state->queue_id,
 	       ATTR_TYPE_END);
     if (attr_scan(src, ATTR_FLAG_STRICT,
-		  ATTR_TYPE_NUM, MAIL_ATTR_FLAGS, &flags,
+		  ATTR_TYPE_INT, MAIL_ATTR_FLAGS, &flags,
 		  ATTR_TYPE_END) != 1) {
 	state->errs |= CLEANUP_STAT_BAD;
 	flags = 0;
@@ -368,7 +440,14 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
      * extracted from message headers.
      */
     while (CLEANUP_OUT_OK(state)) {
-	if ((type = rec_get(src, buf, 0)) < 0) {
+	if ((type = rec_get_raw(src, buf, 0, REC_FLAG_NONE)) < 0) {
+	    state->errs |= CLEANUP_STAT_BAD;
+	    break;
+	}
+	if (type == REC_TYPE_PTR || type == REC_TYPE_DTXT
+	    || type == REC_TYPE_DRCP) {
+	    msg_warn("%s: record type %d not allowed - discarding this message",
+		     state->queue_id, type);
 	    state->errs |= CLEANUP_STAT_BAD;
 	    break;
 	}
@@ -397,8 +476,9 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
     /*
      * Finish this message, and report the result status to the client.
      */
+    status = cleanup_flush(state);		/* in case state is modified */
     attr_print(src, ATTR_FLAG_NONE,
-	       ATTR_TYPE_NUM, MAIL_ATTR_STATUS, cleanup_flush(state),
+	       ATTR_TYPE_INT, MAIL_ATTR_STATUS, status,
 	       ATTR_TYPE_STR, MAIL_ATTR_WHY, state->reason ?
 	       state->reason : "",
 	       ATTR_TYPE_END);
@@ -410,14 +490,6 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
     vstring_free(buf);
 }
 
-/* cleanup_sig - cleanup after signal */
-
-static void cleanup_sig(int sig)
-{
-    cleanup_all();
-    exit(sig);
-}
-
 /* pre_accept - see if tables have changed */
 
 static void pre_accept(char *unused_name, char **unused_argv)
