@@ -6,7 +6,8 @@
 /* SYNOPSIS
 /*	\fBflush\fR [generic Postfix daemon options]
 /* DESCRIPTION
-/*	The flush server maintains a record of deferred mail by destination.
+/*	The \fBflush\fR(8) server maintains a record of deferred
+/*	mail by destination.
 /*	This information is used to improve the performance of the SMTP
 /*	\fBETRN\fR request, and of its command-line equivalent,
 /*	"\fBsendmail -qR\fR" or "\fBpostqueue -f\fR".
@@ -27,7 +28,7 @@
 /*
 /*	This server implements the following requests:
 /* .IP "\fBadd\fI sitename queueid\fR"
-/*	Inform the fast flush server that the message with the specified
+/*	Inform the \fBflush\fR(8) server that the message with the specified
 /*	queue ID is queued for the specified destination.
 /* .IP "\fBsend\fI sitename\fR"
 /*	Request delivery of mail that is queued for the specified
@@ -46,7 +47,7 @@
 /* SECURITY
 /* .ad
 /* .fi
-/*	The fast flush server is not security-sensitive. It does not
+/*	The \fBflush\fR(8) server is not security-sensitive. It does not
 /*	talk to the network, and it does not talk to local users.
 /*	The fast flush server can run chrooted at fixed low privilege.
 /* DIAGNOSTICS
@@ -60,7 +61,7 @@
 /*	\fBmaster.cf\fR configuration file.
 /*
 /*	Upon receipt of a request to deliver mail for an eligible
-/*	destination, the \fBflush\fR server requests delivery of all messages
+/*	destination, the \fBflush\fR(8) server requests delivery of all messages
 /*	that are listed in that destination's logfile, regardless of the
 /*	recipients of those messages. This is not an issue for mail
 /*	that is sent to a \fBrelay_domains\fR destination because
@@ -68,12 +69,12 @@
 /* CONFIGURATION PARAMETERS
 /* .ad
 /* .fi
-/*	Changes to \fBmain.cf\fR are picked up automatically as flush(8)
+/*	Changes to \fBmain.cf\fR are picked up automatically as \fBflush\fR(8)
 /*	processes run for only a limited amount of time. Use the command
 /*	"\fBpostfix reload\fR" to speed up a change.
 /*
 /*	The text below provides only a parameter summary. See
-/*	postconf(5) for more details including examples.
+/*	\fBpostconf\fR(5) for more details including examples.
 /* .IP "\fBconfig_directory (see 'postconf -d' output)\fR"
 /*	The default location of the Postfix main.cf and master.cf
 /*	configuration files.
@@ -266,7 +267,7 @@ static int flush_policy_ok(const char *site)
 
 static int flush_add_service(const char *site, const char *queue_id)
 {
-    char   *myname = "flush_add_service";
+    const char *myname = "flush_add_service";
     VSTRING *site_path;
     int     status;
 
@@ -293,7 +294,7 @@ static int flush_add_service(const char *site, const char *queue_id)
 
 static int flush_add_path(const char *path, const char *queue_id)
 {
-    char   *myname = "flush_add_path";
+    const char *myname = "flush_add_path";
     VSTREAM *log;
 
     /*
@@ -342,7 +343,7 @@ static int flush_add_path(const char *path, const char *queue_id)
 
 static int flush_send_service(const char *site, int how)
 {
-    char   *myname = "flush_send_service";
+    const char *myname = "flush_send_service";
     VSTRING *site_path;
     int     status;
 
@@ -502,7 +503,7 @@ static int flush_send_path(const char *path, int how)
 
 static int flush_refresh_service(int max_age)
 {
-    char   *myname = "flush_refresh_service";
+    const char *myname = "flush_refresh_service";
     SCAN_DIR *scan;
     char   *site_path;
     struct stat st;
@@ -630,7 +631,7 @@ static void flush_service(VSTREAM *client_stream, char *unused_service,
 		&& mail_queue_id_ok(STR(queue_id)))
 		status = flush_add_service(lowercase(STR(site)), STR(queue_id));
 	    attr_print(client_stream, ATTR_FLAG_NONE,
-		       ATTR_TYPE_NUM, MAIL_ATTR_STATUS, status,
+		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, status,
 		       ATTR_TYPE_END);
 	} else if (STREQ(STR(request), FLUSH_REQ_SEND)) {
 	    site = vstring_alloc(10);
@@ -640,25 +641,25 @@ static void flush_service(VSTREAM *client_stream, char *unused_service,
 		status = flush_send_service(lowercase(STR(site)),
 					    REFRESH_AND_DELIVER);
 	    attr_print(client_stream, ATTR_FLAG_NONE,
-		       ATTR_TYPE_NUM, MAIL_ATTR_STATUS, status,
+		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, status,
 		       ATTR_TYPE_END);
 	} else if (STREQ(STR(request), FLUSH_REQ_REFRESH)
 		   || STREQ(STR(request), wakeup)) {
 	    attr_print(client_stream, ATTR_FLAG_NONE,
-		       ATTR_TYPE_NUM, MAIL_ATTR_STATUS, FLUSH_STAT_OK,
+		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, FLUSH_STAT_OK,
 		       ATTR_TYPE_END);
 	    vstream_fflush(client_stream);
 	    (void) flush_refresh_service(var_fflush_refresh);
 	} else if (STREQ(STR(request), FLUSH_REQ_PURGE)) {
 	    attr_print(client_stream, ATTR_FLAG_NONE,
-		       ATTR_TYPE_NUM, MAIL_ATTR_STATUS, FLUSH_STAT_OK,
+		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, FLUSH_STAT_OK,
 		       ATTR_TYPE_END);
 	    vstream_fflush(client_stream);
 	    (void) flush_refresh_service(0);
 	}
     } else
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, status,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, status,
 		   ATTR_TYPE_END);
     vstring_free(request);
     if (site)
