@@ -105,7 +105,7 @@ void    flush_init(void)
 
 int     flush_purge(void)
 {
-    char   *myname = "flush_purge";
+    const char *myname = "flush_purge";
     int     status;
 
     if (msg_verbose)
@@ -131,7 +131,7 @@ int     flush_purge(void)
 
 int     flush_refresh(void)
 {
-    char   *myname = "flush_refresh";
+    const char *myname = "flush_refresh";
     int     status;
 
     if (msg_verbose)
@@ -157,7 +157,7 @@ int     flush_refresh(void)
 
 int     flush_send(const char *site)
 {
-    char   *myname = "flush_send";
+    const char *myname = "flush_send";
     int     status;
 
     if (msg_verbose)
@@ -187,7 +187,7 @@ int     flush_send(const char *site)
 
 int     flush_add(const char *site, const char *queue_id)
 {
-    char   *myname = "flush_add";
+    const char *myname = "flush_add";
     int     status;
 
     if (msg_verbose)
