@@ -46,11 +46,13 @@ extern const char *mail_conf_lookup_eval(const char *);
   */
 extern char *get_mail_conf_str(const char *, const char *, int, int);
 extern int get_mail_conf_int(const char *, int, int, int);
+extern long get_mail_conf_long(const char *, long, long, long);
 extern int get_mail_conf_bool(const char *, int);
 extern int get_mail_conf_time(const char *, const char *, int, int);
 extern char *get_mail_conf_raw(const char *, const char *, int, int);
 
 extern int get_mail_conf_int2(const char *, const char *, int, int, int);
+extern long get_mail_conf_long2(const char *, const char *, long, long, long);
 extern int get_mail_conf_time2(const char *, const char *, const char *, int, int);
 
  /*
@@ -58,6 +60,7 @@ extern int get_mail_conf_time2(const char *, const char *, const char *, int, in
   */
 extern char *get_mail_conf_str_fn(const char *, const char *(*) (void), int, int);
 extern int get_mail_conf_int_fn(const char *, int (*) (void), int, int);
+extern long get_mail_conf_long_fn(const char *, long (*) (void), long, long);
 extern int get_mail_conf_bool_fn(const char *, int (*) (void));
 extern int get_mail_conf_time_fn(const char *, const char *(*) (void), int, int, int);
 extern char *get_mail_conf_raw_fn(const char *, const char *(*) (void), int, int);
@@ -67,6 +70,7 @@ extern char *get_mail_conf_raw_fn(const char *, const char *(*) (void), int, int
   */
 extern void set_mail_conf_str(const char *, const char *);
 extern void set_mail_conf_int(const char *, int);
+extern void set_mail_conf_long(const char *, long);
 extern void set_mail_conf_bool(const char *, int);
 extern void set_mail_conf_time(const char *, const char *);
 
@@ -88,7 +92,7 @@ typedef struct {
     char  **target;			/* pointer to global variable */
     int     min;			/* min length or zero */
     int     max;			/* max length or zero */
-} CONFIG_RAW_TABLE;
+}       CONFIG_RAW_TABLE;
 
 typedef struct {
     const char *name;			/* config variable name */
@@ -98,6 +102,14 @@ typedef struct {
     int     max;			/* upper bound or zero */
 } CONFIG_INT_TABLE;
 
+typedef struct {
+    const char *name;			/* config variable name */
+    long    defval;			/* default value */
+    long   *target;			/* pointer to global variable */
+    long    min;			/* lower bound or zero */
+    long    max;			/* upper bound or zero */
+} CONFIG_LONG_TABLE;
+
 typedef struct {
     const char *name;			/* config variable name */
     int     defval;			/* default value */
@@ -114,6 +126,7 @@ typedef struct {
 
 extern void get_mail_conf_str_table(CONFIG_STR_TABLE *);
 extern void get_mail_conf_int_table(CONFIG_INT_TABLE *);
+extern void get_mail_conf_long_table(CONFIG_LONG_TABLE *);
 extern void get_mail_conf_bool_table(CONFIG_BOOL_TABLE *);
 extern void get_mail_conf_time_table(CONFIG_TIME_TABLE *);
 extern void get_mail_conf_raw_table(CONFIG_RAW_TABLE *);
@@ -136,7 +149,7 @@ typedef struct {
     char  **target;			/* pointer to global variable */
     int     min;			/* lower bound or zero */
     int     max;			/* upper bound or zero */
-} CONFIG_RAW_FN_TABLE;
+}       CONFIG_RAW_FN_TABLE;
 
 typedef struct {
     const char *name;			/* config variable name */
@@ -146,6 +159,14 @@ typedef struct {
     int     max;			/* upper bound or zero */
 } CONFIG_INT_FN_TABLE;
 
+typedef struct {
+    const char *name;			/* config variable name */
+    long    (*defval) (void);		/* default value provider */
+    long   *target;			/* pointer to global variable */
+    long    min;			/* lower bound or zero */
+    long    max;			/* upper bound or zero */
+} CONFIG_LONG_FN_TABLE;
+
 typedef struct {
     const char *name;			/* config variable name */
     int     (*defval) (void);		/* default value provider */
@@ -154,6 +175,7 @@ typedef struct {
 
 extern void get_mail_conf_str_fn_table(CONFIG_STR_FN_TABLE *);
 extern void get_mail_conf_int_fn_table(CONFIG_INT_FN_TABLE *);
+extern void get_mail_conf_long_fn_table(CONFIG_LONG_FN_TABLE *);
 extern void get_mail_conf_bool_fn_table(CONFIG_BOOL_FN_TABLE *);
 extern void get_mail_conf_raw_fn_table(CONFIG_RAW_FN_TABLE *);
 
