@@ -86,6 +86,7 @@
 #include <inet_addr_list.h>
 #include <host_port.h>
 #include <inet_addr_host.h>
+#include <sock_addr.h>
 
 /* Global library. */
 
@@ -121,7 +122,7 @@ void    fset_master_ent(char *path)
 
 void    set_master_ent()
 {
-    char   *myname = "set_master_ent";
+    const char *myname = "set_master_ent";
 
     if (master_fp != 0)
 	msg_panic("%s: configuration file still open", myname);
@@ -136,7 +137,7 @@ void    set_master_ent()
 
 void    end_master_ent()
 {
-    char   *myname = "end_master_ent";
+    const char *myname = "end_master_ent";
 
     if (master_fp == 0)
 	msg_panic("%s: configuration file not open", myname);
@@ -149,7 +150,7 @@ void    end_master_ent()
 
 static NORETURN fatal_with_context(char *format,...)
 {
-    char   *myname = "fatal_with_context";
+    const char *myname = "fatal_with_context";
     VSTRING *vp = vstring_alloc(100);
     va_list ap;
 
@@ -225,8 +226,8 @@ MASTER_SERV *get_master_ent()
     MASTER_SERV *serv;
     char   *cp;
     char   *name;
-    char   *host;
-    char   *port;
+    char   *host = 0;
+    char   *port = 0;
     char   *transport;
     int     private;
     int     unprivileged;		/* passed on to child */
@@ -280,9 +281,15 @@ MASTER_SERV *get_master_ent()
      */
     serv->flags = 0;
 
+    /*
+     * All servers busy warning timer.
+     */
+    serv->busy_warn_time = 0;
+
     /*
      * Service name. Syntax is transport-specific.
      */
+    serv->ext_name = mystrdup(cp);
     name = cp;
 
     /*
@@ -294,7 +301,7 @@ MASTER_SERV *get_master_ent()
     if (STR_SAME(transport, MASTER_XPORT_NAME_INET)) {
 	if (!STR_SAME(saved_interfaces, var_inet_interfaces)) {
 	    msg_warn("service %s: ignoring %s change",
-		     name, VAR_INET_INTERFACES);
+		     serv->ext_name, VAR_INET_INTERFACES);
 	    msg_warn("to change %s, stop and start Postfix",
 		     VAR_INET_INTERFACES);
 	}
@@ -323,17 +330,27 @@ MASTER_SERV *get_master_ent()
 	    serv->listen_fd_count = MASTER_INET_ADDRLIST(serv)->used;
 	}
 	MASTER_INET_PORT(serv) = mystrdup(port);
-	myfree(atmp);
+	for (n = 0; /* see below */ ; n++) {
+	    if (n >= MASTER_INET_ADDRLIST(serv)->used) {
+		serv->flags |= MASTER_FLAG_LOCAL_ONLY;
+		break;
+	    }
+	    if (!sock_addr_in_loopback(SOCK_ADDR_PTR(MASTER_INET_ADDRLIST(serv)->addrs + n)))
+		break;
+	}
     } else if (STR_SAME(transport, MASTER_XPORT_NAME_UNIX)) {
 	serv->type = MASTER_SERV_TYPE_UNIX;
 	serv->listen_fd_count = 1;
+	serv->flags |= MASTER_FLAG_LOCAL_ONLY;
     } else if (STR_SAME(transport, MASTER_XPORT_NAME_FIFO)) {
 	serv->type = MASTER_SERV_TYPE_FIFO;
 	serv->listen_fd_count = 1;
+	serv->flags |= MASTER_FLAG_LOCAL_ONLY;
 #ifdef MASTER_SERV_TYPE_PASS
     } else if (STR_SAME(transport, MASTER_XPORT_NAME_PASS)) {
 	serv->type = MASTER_SERV_TYPE_PASS;
 	serv->listen_fd_count = 1;
+	/* If this is a connection screener, remote clients are likely. */
 #endif
     } else {
 	fatal_with_context("bad transport type: %s", transport);
@@ -349,9 +366,42 @@ MASTER_SERV *get_master_ent()
      * attributes such as privacy.
      */
     if (serv->type == MASTER_SERV_TYPE_INET) {
+	MAI_HOSTADDR_STR host_addr;
+	MAI_SERVPORT_STR serv_port;
+	struct addrinfo *res0;
+
 	if (private)
 	    fatal_with_context("inet service cannot be private");
-	serv->name = mystrdup(name);
+
+	/*
+	 * Canonicalize endpoint names so that we correctly handle "reload"
+	 * requests after someone changes "25" into "smtp" or vice versa.
+	 */
+	if (*host == 0)
+	    host = 0;
+	/* Canonicalize numeric host and numeric or symbolic service. */
+	if (hostaddr_to_sockaddr(host, port, 0, &res0) == 0) {
+	    SOCKADDR_TO_HOSTADDR(res0->ai_addr, res0->ai_addrlen,
+				 host ? &host_addr : (MAI_HOSTADDR_STR *) 0,
+				 &serv_port, 0);
+	    serv->name = (host ? concatenate("[", host_addr.buf, "]:",
+					     serv_port.buf, (char *) 0) :
+			  mystrdup(serv_port.buf));
+	    freeaddrinfo(res0);
+	}
+	/* Canonicalize numeric or symbolic service. */
+	else if (hostaddr_to_sockaddr((char *) 0, port, 0, &res0) == 0) {
+	    SOCKADDR_TO_HOSTADDR(res0->ai_addr, res0->ai_addrlen,
+				 (MAI_HOSTADDR_STR *) 0, &serv_port, 0);
+	    serv->name = (host ? concatenate("[", host, "]:",
+					     serv_port.buf, (char *) 0) :
+			  mystrdup(serv_port.buf));
+	    freeaddrinfo(res0);
+	}
+	/* Bad service name? */
+	else
+	    serv->name = mystrdup(name);
+	myfree(atmp);
     } else if (serv->type == MASTER_SERV_TYPE_UNIX) {
 	serv->name = mail_pathname(private ? MAIL_CLASS_PRIVATE :
 				   MAIL_CLASS_PUBLIC, name);
@@ -453,6 +503,8 @@ MASTER_SERV *get_master_ent()
     if (strcmp(basename(command), name) != 0)
 	argv_add(serv->args, "-n", name, (char *) 0);
     argv_add(serv->args, "-t", transport, (char *) 0);
+    if (master_detach == 0)
+	argv_add(serv->args, "-d", (char *) 0);
     if (msg_verbose)
 	argv_add(serv->args, "-v", (char *) 0);
     if (unprivileged)
@@ -521,6 +573,7 @@ void    free_master_ent(MASTER_SERV *serv)
     }
     if (serv->type == MASTER_SERV_TYPE_INET)
 	myfree(MASTER_INET_PORT(serv));
+    myfree(serv->ext_name);
     myfree(serv->name);
     myfree(serv->path);
     argv_free(serv->args);
