@@ -55,6 +55,8 @@
 #include <vstring_vstream.h>
 #include <events.h>
 #include <iostuff.h>
+#include <stringops.h>
+#include <mymalloc.h>
 
 /* Global library. */
 
@@ -64,6 +66,10 @@
 #include <mail_params.h>
 #include <deliver_request.h>
 #include <verp_sender.h>
+#include <dsn_util.h>
+#include <dsn_buf.h>
+#include <dsb_scan.h>
+#include <rcpt_print.h>
 
 /* Application-specific. */
 
@@ -88,7 +94,7 @@ static int qmgr_deliver_initial_reply(VSTREAM *stream)
 	msg_warn("%s: premature disconnect", VSTREAM_PATH(stream));
 	return (DELIVER_STAT_CRASH);
     } else if (attr_scan(stream, ATTR_FLAG_STRICT,
-			 ATTR_TYPE_NUM, MAIL_ATTR_STATUS, &stat,
+			 ATTR_TYPE_INT, MAIL_ATTR_STATUS, &stat,
 			 ATTR_TYPE_END) != 1) {
 	msg_warn("%s: malformed response", VSTREAM_PATH(stream));
 	return (DELIVER_STAT_CRASH);
@@ -99,7 +105,7 @@ static int qmgr_deliver_initial_reply(VSTREAM *stream)
 
 /* qmgr_deliver_final_reply - retrieve final delivery process response */
 
-static int qmgr_deliver_final_reply(VSTREAM *stream, VSTRING *reason)
+static int qmgr_deliver_final_reply(VSTREAM *stream, DSN_BUF *dsb)
 {
     int     stat;
 
@@ -107,8 +113,8 @@ static int qmgr_deliver_final_reply(VSTREAM *stream, VSTRING *reason)
 	msg_warn("%s: premature disconnect", VSTREAM_PATH(stream));
 	return (DELIVER_STAT_CRASH);
     } else if (attr_scan(stream, ATTR_FLAG_STRICT,
-			 ATTR_TYPE_STR, MAIL_ATTR_WHY, reason,
-			 ATTR_TYPE_NUM, MAIL_ATTR_STATUS, &stat,
+			 ATTR_TYPE_FUNC, dsb_scan, (void *) dsb,
+			 ATTR_TYPE_INT, MAIL_ATTR_STATUS, &stat,
 			 ATTR_TYPE_END) != 2) {
 	msg_warn("%s: malformed response", VSTREAM_PATH(stream));
 	return (DELIVER_STAT_CRASH);
@@ -121,10 +127,11 @@ static int qmgr_deliver_final_reply(VSTREAM *stream, VSTRING *reason)
 
 static int qmgr_deliver_send_request(QMGR_ENTRY *entry, VSTREAM *stream)
 {
-    QMGR_RCPT_LIST list = entry->rcpt_list;
-    QMGR_RCPT *recipient;
+    RECIPIENT_LIST list = entry->rcpt_list;
+    RECIPIENT *recipient;
     QMGR_MESSAGE *message = entry->message;
     VSTRING *sender_buf = 0;
+    MSG_STATS stats;
     char   *sender;
     int     flags;
 
@@ -145,8 +152,9 @@ static int qmgr_deliver_send_request(QMGR_ENTRY *entry, VSTREAM *stream)
     flags = message->tflags
 	| entry->queue->dflags
 	| (message->inspect_xport ? DEL_REQ_FLAG_BOUNCE : DEL_REQ_FLAG_DEFLT);
-    attr_print(stream, ATTR_FLAG_MORE,
-	       ATTR_TYPE_NUM, MAIL_ATTR_FLAGS, flags,
+    (void) QMGR_MSG_STATS(&stats, message);
+    attr_print(stream, ATTR_FLAG_NONE,
+	       ATTR_TYPE_INT, MAIL_ATTR_FLAGS, flags,
 	       ATTR_TYPE_STR, MAIL_ATTR_QUEUE, message->queue_name,
 	       ATTR_TYPE_STR, MAIL_ATTR_QUEUEID, message->queue_id,
 	       ATTR_TYPE_LONG, MAIL_ATTR_OFFSET, message->data_offset,
@@ -154,29 +162,25 @@ static int qmgr_deliver_send_request(QMGR_ENTRY *entry, VSTREAM *stream)
 	       ATTR_TYPE_STR, MAIL_ATTR_NEXTHOP, entry->queue->nexthop,
 	       ATTR_TYPE_STR, MAIL_ATTR_ENCODING, message->encoding,
 	       ATTR_TYPE_STR, MAIL_ATTR_SENDER, sender,
-	       ATTR_TYPE_STR, MAIL_ATTR_ERRTO, message->errors_to,
-	       ATTR_TYPE_STR, MAIL_ATTR_RRCPT, message->return_receipt,
-	       ATTR_TYPE_LONG, MAIL_ATTR_TIME, message->arrival_time,
-	       ATTR_TYPE_STR, MAIL_ATTR_CLIENT_NAME, message->client_name,
-	       ATTR_TYPE_STR, MAIL_ATTR_CLIENT_ADDR, message->client_addr,
-	       ATTR_TYPE_STR, MAIL_ATTR_PROTO_NAME, message->client_proto,
-	       ATTR_TYPE_STR, MAIL_ATTR_HELO_NAME, message->client_helo,
+	       ATTR_TYPE_STR, MAIL_ATTR_DSN_ENVID, message->dsn_envid,
+	       ATTR_TYPE_INT, MAIL_ATTR_DSN_RET, message->dsn_ret,
+	       ATTR_TYPE_FUNC, msg_stats_print, (void *) &stats,
+	     ATTR_TYPE_STR, MAIL_ATTR_LOG_CLIENT_NAME, message->client_name,
+	     ATTR_TYPE_STR, MAIL_ATTR_LOG_CLIENT_ADDR, message->client_addr,
+	     ATTR_TYPE_STR, MAIL_ATTR_LOG_PROTO_NAME, message->client_proto,
+	       ATTR_TYPE_STR, MAIL_ATTR_LOG_HELO_NAME, message->client_helo,
 	       ATTR_TYPE_STR, MAIL_ATTR_SASL_METHOD, message->sasl_method,
-	       ATTR_TYPE_STR, MAIL_ATTR_SASL_USERNAME, message->sasl_username,
+	     ATTR_TYPE_STR, MAIL_ATTR_SASL_USERNAME, message->sasl_username,
 	       ATTR_TYPE_STR, MAIL_ATTR_SASL_SENDER, message->sasl_sender,
-	       ATTR_TYPE_STR, MAIL_ATTR_RWR_CONTEXT, message->rewrite_context,
+	     ATTR_TYPE_STR, MAIL_ATTR_RWR_CONTEXT, message->rewrite_context,
+	       ATTR_TYPE_INT, MAIL_ATTR_RCPT_COUNT, list.len,
 	       ATTR_TYPE_END);
     if (sender_buf != 0)
 	vstring_free(sender_buf);
     for (recipient = list.info; recipient < list.info + list.len; recipient++)
-	attr_print(stream, ATTR_FLAG_MORE,
-		   ATTR_TYPE_LONG, MAIL_ATTR_OFFSET, recipient->offset,
-		   ATTR_TYPE_STR, MAIL_ATTR_ORCPT, recipient->orig_rcpt,
-		   ATTR_TYPE_STR, MAIL_ATTR_RECIP, recipient->address,
+	attr_print(stream, ATTR_FLAG_NONE,
+		   ATTR_TYPE_FUNC, rcpt_print, (void *) recipient,
 		   ATTR_TYPE_END);
-    attr_print(stream, ATTR_FLAG_NONE,
-	       ATTR_TYPE_NUM, MAIL_ATTR_OFFSET, 0,
-	       ATTR_TYPE_END);
     if (vstream_fflush(stream) != 0) {
 	msg_warn("write to process (%s): %m", entry->queue->transport->name);
 	return (-1);
@@ -208,8 +212,13 @@ static void qmgr_deliver_update(int unused_event, char *context)
     QMGR_QUEUE *queue = entry->queue;
     QMGR_TRANSPORT *transport = queue->transport;
     QMGR_MESSAGE *message = entry->message;
-    VSTRING *reason = vstring_alloc(1);
+    static DSN_BUF *dsb;
     int     status;
+    RECIPIENT *recipient;
+    int     nrcpt;
+
+    if (dsb == 0)
+	dsb = dsb_create();
 
     /*
      * The message transport has responded. Stop the watchdog timer.
@@ -223,7 +232,7 @@ static void qmgr_deliver_update(int unused_event, char *context)
      * manager can log why it does not even try to schedule delivery to the
      * affected recipients.
      */
-    status = qmgr_deliver_final_reply(entry->stream, reason);
+    status = qmgr_deliver_final_reply(entry->stream, dsb);
 
     /*
      * The mail delivery process failed for some reason (although delivery
@@ -236,10 +245,35 @@ static void qmgr_deliver_update(int unused_event, char *context)
      */
     if (status == DELIVER_STAT_CRASH) {
 	message->flags |= DELIVER_STAT_DEFER;
-	qmgr_transport_throttle(transport, "unknown mail transport error");
+#if 0
+	whatsup = concatenate("unknown ", transport->name,
+			      " mail transport error", (char *) 0);
+	qmgr_transport_throttle(transport,
+				DSN_SIMPLE(&dsb->dsn, "4.3.0", whatsup));
+	myfree(whatsup);
+#else
+	qmgr_transport_throttle(transport,
+				DSN_SIMPLE(&dsb->dsn, "4.3.0",
+					   "unknown mail transport error"));
+#endif
 	msg_warn("transport %s failure -- see a previous warning/fatal/panic logfile record for the problem description",
 		 transport->name);
-	qmgr_defer_transport(transport, transport->reason);
+
+	/*
+	 * Assume the worst and write a defer logfile record for each
+	 * recipient. This omission was already present in the first queue
+	 * manager implementation of 199703, and was fixed 200511.
+	 * 
+	 * Don't move this queue entry back to the todo queue so that
+	 * qmgr_defer_transport() can update the defer log. The queue entry
+	 * is still hot, and making it cold would involve duplicating most
+	 * but not all code at the end of this routine. That's too tricky.
+	 */
+	for (nrcpt = 0; nrcpt < entry->rcpt_list.len; nrcpt++) {
+	    recipient = entry->rcpt_list.info + nrcpt;
+	    qmgr_defer_recipient(message, recipient, &dsb->dsn);
+	}
+	qmgr_defer_transport(transport, &dsb->dsn);
     }
 
     /*
@@ -250,13 +284,23 @@ static void qmgr_deliver_update(int unused_event, char *context)
      * (the todo list); stay away from queue entries that have been selected
      * (the busy list), or we would have dangling pointers. The queue itself
      * won't go away before we dispose of the current queue entry.
+     * 
+     * XXX Caution: DSN_COPY() will panic on empty status or reason.
      */
+#define SUSPENDED	"delivery temporarily suspended: "
+
     if (status == DELIVER_STAT_DEFER) {
 	message->flags |= DELIVER_STAT_DEFER;
-	if (VSTRING_LEN(reason)) {
-	    qmgr_queue_throttle(queue, vstring_str(reason));
+	if (VSTRING_LEN(dsb->status)) {
+	    /* Sanitize the DSN status/reason from the delivery agent. */
+	    if (!dsn_valid(vstring_str(dsb->status)))
+		vstring_strcpy(dsb->status, "4.0.0");
+	    if (VSTRING_LEN(dsb->reason) == 0)
+		vstring_strcpy(dsb->reason, "unknown error");
+	    vstring_prepend(dsb->reason, SUSPENDED, sizeof(SUSPENDED) - 1);
+	    qmgr_queue_throttle(queue, DSN_FROM_DSN_BUF(dsb));
 	    if (queue->window == 0)
-		qmgr_defer_todo(queue, queue->reason);
+		qmgr_defer_todo(queue, &dsb->dsn);
 	}
     }
 
@@ -264,7 +308,7 @@ static void qmgr_deliver_update(int unused_event, char *context)
      * No problems detected. Mark the transport and queue as alive. The queue
      * itself won't go away before we dispose of the current queue entry.
      */
-    if (VSTRING_LEN(reason) == 0) {
+    if (status != DELIVER_STAT_CRASH && VSTRING_LEN(dsb->reason) == 0) {
 	qmgr_transport_unthrottle(transport);
 	qmgr_queue_unthrottle(queue);
     }
@@ -280,7 +324,6 @@ static void qmgr_deliver_update(int unused_event, char *context)
     entry->stream = 0;
     qmgr_deliver_concurrency--;
     qmgr_entry_done(entry, QMGR_QUEUE_BUSY);
-    vstring_free(reason);
 }
 
 /* qmgr_deliver - deliver one per-site queue entry */
@@ -289,6 +332,7 @@ void    qmgr_deliver(QMGR_TRANSPORT *transport, VSTREAM *stream)
 {
     QMGR_QUEUE *queue;
     QMGR_ENTRY *entry;
+    DSN     dsn;
 
     /*
      * Find out if this delivery process is really available. Once elected,
@@ -298,8 +342,18 @@ void    qmgr_deliver(QMGR_TRANSPORT *transport, VSTREAM *stream)
      * while some other queue manipulation is happening.
      */
     if (qmgr_deliver_initial_reply(stream) != 0) {
-	qmgr_transport_throttle(transport, "mail transport unavailable");
-	qmgr_defer_transport(transport, transport->reason);
+#if 0
+	whatsup = concatenate(transport->name,
+			      " mail transport unavailable", (char *) 0);
+	qmgr_transport_throttle(transport,
+				DSN_SIMPLE(&dsn, "4.3.0", whatsup));
+	myfree(whatsup);
+#else
+	qmgr_transport_throttle(transport,
+				DSN_SIMPLE(&dsn, "4.3.0",
+					   "mail transport unavailable"));
+#endif
+	qmgr_defer_transport(transport, &dsn);
 	(void) vstream_fclose(stream);
 	return;
     }
@@ -325,8 +379,18 @@ void    qmgr_deliver(QMGR_TRANSPORT *transport, VSTREAM *stream)
      */
     if (qmgr_deliver_send_request(entry, stream) < 0) {
 	qmgr_entry_unselect(queue, entry);
-	qmgr_transport_throttle(transport, "mail transport unavailable");
-	qmgr_defer_transport(transport, transport->reason);
+#if 0
+	whatsup = concatenate(transport->name,
+			      " mail transport unavailable", (char *) 0);
+	qmgr_transport_throttle(transport,
+				DSN_SIMPLE(&dsn, "4.3.0", whatsup));
+	myfree(whatsup);
+#else
+	qmgr_transport_throttle(transport,
+				DSN_SIMPLE(&dsn, "4.3.0",
+					   "mail transport unavailable"));
+#endif
+	qmgr_defer_transport(transport, &dsn);
 	/* warning: entry and queue may be dangling pointers here */
 	(void) vstream_fclose(stream);
 	return;
