@@ -6,10 +6,10 @@
 /* SYNOPSIS
 /*	#include "qmgr.h"
 /*
-/*	QMGR_QUEUE *qmgr_bounce_recipient(message, recipient, format, ...)
+/*	QMGR_QUEUE *qmgr_bounce_recipient(message, recipient, dsn)
 /*	QMGR_MESSAGE *message;
-/*	QMGR_RCPT *recipient;
-/*	const char *format;
+/*	RECIPIENT *recipient;
+/*	DSN	*dsn;
 /* DESCRIPTION
 /*	qmgr_bounce_recipient() produces a bounce log record.
 /*	Once the bounce record is written successfully, the recipient
@@ -22,8 +22,8 @@
 /*	Open queue file with the message being bounced.
 /* .IP recipient
 /*	The recipient that will not be delivered.
-/* .IP format
-/*	Free-format text that describes why delivery will not happen.
+/* .IP dsn
+/*	Delivery status information. See dsn(3).
 /* DIAGNOSTICS
 /*	Panic: consistency check failure. Fatal: out of memory.
 /* LICENSE
@@ -45,7 +45,6 @@
 /* System library. */
 
 #include <sys_defs.h>
-#include <stdarg.h>
 
 /* Utility library. */
 
@@ -60,18 +59,15 @@
 
 /* qmgr_bounce_recipient - bounce one message recipient */
 
-void    qmgr_bounce_recipient(QMGR_MESSAGE *message, QMGR_RCPT *recipient,
-			              const char *format,...)
+void    qmgr_bounce_recipient(QMGR_MESSAGE *message, RECIPIENT *recipient,
+			              DSN *dsn)
 {
-    va_list ap;
+    MSG_STATS stats;
     int     status;
 
-    va_start(ap, format);
-    status = vbounce_append(message->tflags, message->queue_id,
-			    recipient->orig_rcpt, recipient->address,
-			    recipient->offset, "none", message->arrival_time,
-			    format, ap);
-    va_end(ap);
+    status = bounce_append(message->tflags, message->queue_id,
+			   QMGR_MSG_STATS(&stats, message), recipient,
+			   "none", dsn);
 
     if (status == 0)
 	deliver_completed(message->fp, recipient->offset);
