@@ -11,22 +11,16 @@
  /*
   * System library.
   */
+#include <sys/time.h>
 #include <unistd.h>
 
- /*
-  * SASL library.
-  */
-#ifdef USE_SASL_AUTH
-#include <sasl.h>
-#include <saslutil.h>
-#endif
-
  /*
   * Utility library.
   */
 #include <vstream.h>
 #include <vstring.h>
 #include <argv.h>
+#include <myaddrinfo.h>
 
  /*
   * Global library.
@@ -36,9 +30,12 @@
  /*
   * Postfix TLS library.
   */
-#ifdef USE_TLS
 #include <tls.h>
-#endif
+
+ /*
+  * Milter library.
+  */
+#include <milter.h>
 
  /*
   * Variables that keep track of conversation state. There is only one SMTP
@@ -49,6 +46,8 @@
 typedef struct SMTPD_DEFER {
     int     active;			/* is this active */
     VSTRING *reason;			/* reason for deferral */
+    VSTRING *dsn;			/* DSN detail */
+    int     code;			/* SMTP reply code */
     int     class;			/* error notification class */
 } SMTPD_DEFER;
 
@@ -65,16 +64,24 @@ typedef struct {
 } SMTPD_XFORWARD_ATTR;
 
 typedef struct SMTPD_STATE {
+    int     flags;			/* see below */
     int     err;			/* cleanup server/queue file errors */
     VSTREAM *client;			/* SMTP client handle */
     VSTRING *buffer;			/* SMTP client buffer */
+    VSTRING *addr_buf;			/* internalized address buffer */
     char   *service;			/* for event rate control */
-    time_t  time;			/* start of MAIL FROM transaction */
-    char   *name;			/* client hostname */
+    struct timeval arrival_time;	/* start of MAIL FROM transaction */
+    char   *name;			/* verified client hostname */
+    char   *reverse_name;		/* unverified client hostname */
     char   *addr;			/* client host address string */
     char   *namaddr;			/* combined name and address */
     char   *rfc_addr;			/* address for RFC 2821 */
-    int     peer_code;			/* 2=ok, 4=soft, 5=hard */
+    int     addr_family;		/* address family */
+    struct sockaddr_storage sockaddr;	/* binary client endpoint */
+    int     name_status;		/* 2=ok 4=soft 5=hard 6=forged */
+    int     reverse_name_status;	/* 2=ok 4=soft 5=hard */
+    int     conn_count;			/* connections from this client */
+    int     conn_rate;			/* connection rate for this client */
     int     error_count;		/* reset after DOT */
     int     error_mask;			/* client errors */
     int     notify_mask;		/* what to report to postmaster */
@@ -104,17 +111,12 @@ typedef struct SMTPD_STATE {
      * SASL specific.
      */
 #ifdef USE_SASL_AUTH
-#if SASL_VERSION_MAJOR >= 2
-    const char *sasl_mechanism_list;
-#else
+    struct XSASL_SERVER *sasl_server;
+    VSTRING *sasl_reply;
     char   *sasl_mechanism_list;
-#endif
     char   *sasl_method;
     char   *sasl_username;
     char   *sasl_sender;
-    sasl_conn_t *sasl_conn;
-    VSTRING *sasl_encoded;
-    VSTRING *sasl_decoded;
 #endif
 
     /*
@@ -132,10 +134,18 @@ typedef struct SMTPD_STATE {
     char   *saved_filter;		/* postponed filter action */
     char   *saved_redirect;		/* postponed redirect action */
     int     saved_flags;		/* postponed hold/discard */
+#ifdef DELAY_ACTION
+    int     saved_delay;		/* postponed deferred delay */
+#endif
     VSTRING *expand_buf;		/* scratch space for $name expansion */
     ARGV   *prepend;			/* prepended headers */
     VSTRING *instance;			/* policy query correlation */
     int     seqno;			/* policy query correlation */
+    int     ehlo_discard_mask;		/* suppressed EHLO features */
+    char   *dsn_envid;			/* temporary MAIL FROM state */
+    int     dsn_ret;			/* temporary MAIL FROM state */
+    VSTRING *dsn_buf;			/* scratch space for xtext expansion */
+    VSTRING *dsn_orcpt_buf;		/* scratch space for ORCPT parsing */
 
     /*
      * Pass-through proxy client.
@@ -158,11 +168,17 @@ typedef struct SMTPD_STATE {
     int     tls_enforce_tls;		/* must use TLS */
     int     tls_auth_only;		/* use SASL over TLS only */
     TLScontext_t *tls_context;		/* TLS session state */
-    tls_info_t tls_info;		/* legacy */
 #endif
 
+    /*
+     * Milter support.
+     */
+    const char **milter_argv;
+    ssize_t milter_argc;
 } SMTPD_STATE;
 
+#define SMTPD_FLAG_HANGUP	(1<<0)	/* disconnect */
+
 #define SMTPD_STATE_XFORWARD_INIT  (1<<0)	/* xforward preset done */
 #define SMTPD_STATE_XFORWARD_NAME  (1<<1)	/* client name received */
 #define SMTPD_STATE_XFORWARD_ADDR  (1<<2)	/* client address received */
@@ -185,6 +201,27 @@ extern void smtpd_state_reset(SMTPD_STATE *);
 #define SMTPD_AFTER_CONNECT	"CONNECT"
 #define SMTPD_AFTER_DOT		"END-OF-MESSAGE"
 
+ /*
+  * Other stages. These are sometimes used to change the way information is
+  * logged or what information will be available for access control.
+  */
+#define SMTPD_CMD_HELO		"HELO"
+#define SMTPD_CMD_EHLO		"EHLO"
+#define SMTPD_CMD_STARTTLS	"STARTTLS"
+#define SMTPD_CMD_AUTH		"AUTH"
+#define SMTPD_CMD_MAIL		"MAIL"
+#define SMTPD_CMD_RCPT		"RCPT"
+#define SMTPD_CMD_DATA		"DATA"
+#define SMTPD_CMD_EOD		SMTPD_AFTER_DOT	/* XXX Was: END-OF-DATA */
+#define SMTPD_CMD_RSET		"RSET"
+#define SMTPD_CMD_NOOP		"NOOP"
+#define SMTPD_CMD_VRFY		"VRFY"
+#define SMTPD_CMD_ETRN		"ETRN"
+#define SMTPD_CMD_QUIT		"QUIT"
+#define SMTPD_CMD_XCLIENT	"XCLIENT"
+#define SMTPD_CMD_XFORWARD	"XFORWARD"
+#define SMTPD_CMD_UNKNOWN	"UNKNOWN"
+
  /*
   * Representation of unknown client information within smtpd processes. This
   * is not the representation that Postfix uses in queue files, in queue
@@ -233,6 +270,7 @@ extern void smtpd_peer_reset(SMTPD_STATE *state);
 #define	SMTPD_PEER_CODE_OK	2
 #define SMTPD_PEER_CODE_TEMP	4
 #define SMTPD_PEER_CODE_PERM	5
+#define SMTPD_PEER_CODE_FORGED	6
 
  /*
   * Choose between normal or forwarded attributes.
@@ -279,6 +317,11 @@ extern void smtpd_xforward_reset(SMTPD_STATE *);
   */
 extern int smtpd_input_transp_mask;
 
+ /*
+  * More Milter support.
+  */
+extern MILTERS *smtpd_milters;
+
 /* LICENSE
 /* .ad
 /* .fi
