@@ -17,7 +17,7 @@
 /*	int	delay;
 /*
 /*	int	event_cancel_timer(callback, context)
-/*	void	(*callback)(char *context);
+/*	void	(*callback)(int event, char *context);
 /*	char	*context;
 /*
 /*	void	event_enable_read(fd, callback, context)
@@ -63,7 +63,7 @@
 /*	The event argument is equal to EVENT_TIME.
 /*	Only one timer request can be active per (callback, context) pair.
 /*	Calling event_request_timer() with an existing (callback, context)
-/*	pair does not schedule a new event, but updates the moment of
+/*	pair does not schedule a new event, but updates the time of event
 /*	delivery. The result is the absolute time at which the timer is
 /*	scheduled to go off.
 /*
@@ -261,9 +261,10 @@ void    event_drain(int time_limit)
 	return;
 
     FD_ZERO(&zero_mask);
+    (void) time(&event_present);
     max_time = event_present + time_limit;
     while (event_present < max_time
-	   && (event_timer_head.pred != event_timer_head.succ
+	   && (event_timer_head.pred != &event_timer_head
 	       || memcmp(&zero_mask, &event_xmask, sizeof(zero_mask)) != 0))
 	event_loop(1);
 }
@@ -272,7 +273,7 @@ void    event_drain(int time_limit)
 
 void    event_enable_read(int fd, EVENT_NOTIFY_RDWR callback, char *context)
 {
-    char   *myname = "event_enable_read";
+    const char *myname = "event_enable_read";
     EVENT_FDTABLE *fdp;
 
     if (EVENT_INIT_NEEDED())
@@ -311,7 +312,7 @@ void    event_enable_read(int fd, EVENT_NOTIFY_RDWR callback, char *context)
 
 void    event_enable_write(int fd, EVENT_NOTIFY_RDWR callback, char *context)
 {
-    char   *myname = "event_enable_write";
+    const char *myname = "event_enable_write";
     EVENT_FDTABLE *fdp;
 
     if (EVENT_INIT_NEEDED())
@@ -350,7 +351,7 @@ void    event_enable_write(int fd, EVENT_NOTIFY_RDWR callback, char *context)
 
 void    event_disable_readwrite(int fd)
 {
-    char   *myname = "event_disable_readwrite";
+    const char *myname = "event_disable_readwrite";
     EVENT_FDTABLE *fdp;
 
     if (EVENT_INIT_NEEDED())
@@ -381,7 +382,7 @@ void    event_disable_readwrite(int fd)
 
 time_t  event_request_timer(EVENT_NOTIFY_TIME callback, char *context, int delay)
 {
-    char   *myname = "event_request_timer";
+    const char *myname = "event_request_timer";
     RING   *ring;
     EVENT_TIMER *timer;
 
@@ -445,7 +446,7 @@ time_t  event_request_timer(EVENT_NOTIFY_TIME callback, char *context, int delay
 
 int     event_cancel_timer(EVENT_NOTIFY_TIME callback, char *context)
 {
-    char   *myname = "event_cancel_timer";
+    const char *myname = "event_cancel_timer";
     RING   *ring;
     EVENT_TIMER *timer;
     int     time_left = -1;
@@ -478,7 +479,7 @@ int     event_cancel_timer(EVENT_NOTIFY_TIME callback, char *context)
 
 void    event_loop(int delay)
 {
-    char   *myname = "event_loop";
+    const char *myname = "event_loop";
     static int nested;
     fd_set  rmask;
     fd_set  wmask;
@@ -493,6 +494,20 @@ void    event_loop(int delay)
     if (EVENT_INIT_NEEDED())
 	event_init();
 
+    /*
+     * XXX Also print the select() masks?
+     */
+    if (msg_verbose > 2) {
+	RING   *ring;
+
+	FOREACH_QUEUE_ENTRY(ring, &event_timer_head) {
+	    timer = RING_TO_TIMER(ring);
+	    msg_info("%s: time left %3d for 0x%lx 0x%lx", myname,
+		     (int) (timer->when - event_present),
+		     (long) timer->callback, (long) timer->context);
+	}
+    }
+
     /*
      * Find out when the next timer would go off. Timer requests are sorted.
      * If any timer is scheduled, adjust the delay appropriately.
@@ -598,23 +613,20 @@ void    event_loop(int delay)
 #ifdef TEST
 
  /*
-  * Proof-of-concept test program for the event manager. Print "dingdong"
-  * every 5 seconds, while echoing any lines read from stdin. The "dingdong
-  * changes case each time it is printed.
+  * Proof-of-concept test program for the event manager. Schedule a series of
+  * events at one-second intervals and let them happen, while echoing any
+  * lines read from stdin.
   */
 #include <stdio.h>
 #include <ctype.h>
+#include <stdlib.h>
 
-#define DELAY 5
+/* timer_event - display event */
 
-/* dingdong - print text every DELAY seconds */
-
-static void dingdong(char *context)
+static void timer_event(int unused_event, char *context)
 {
-    printf("%c", *context);
+    printf("%ld: %s\n", (long) event_present, context);
     fflush(stdout);
-    *context = (ISUPPER(*context) ? TOLOWER(*context) : TOUPPER(*context));
-    event_request_timer(dingdong, context, (char *) DELAY);
 }
 
 /* echo - echo text received on stdin */
@@ -628,13 +640,14 @@ static void echo(int unused_event, char *unused_context)
     printf("Result: %s", buf);
 }
 
-main(void)
+int     main(void)
 {
-    static char text[] = "\rdingdong ";
-    char   *cp;
-
-    for (cp = text; *cp; cp++)
-	event_request_timer(dingdong, cp, (char *) 0);
+    event_request_timer(timer_event, "3 first", 3);
+    event_request_timer(timer_event, "3 second", 3);
+    event_request_timer(timer_event, "4 first", 4);
+    event_request_timer(timer_event, "4 second", 4);
+    event_request_timer(timer_event, "2 first", 2);
+    event_request_timer(timer_event, "2 second", 2);
     event_enable_read(fileno(stdin), echo, (char *) 0);
     for (;;)
 	event_loop(-1);
