@@ -6,7 +6,7 @@
 /* SYNOPSIS
 /*	\fBverify\fR [generic Postfix daemon options]
 /* DESCRIPTION
-/*	The Postfix address verification server maintains a record
+/*	The \fBverify\fR(8) address verification server maintains a record
 /*	of what recipient addresses are known to be deliverable or
 /*	undeliverable.
 /*
@@ -19,13 +19,14 @@
 /*	MTA for the specified address, and will therefore not detect
 /*	all undeliverable addresses.
 /*
-/*	This server is designed to run under control by the Postfix
+/*	The \fBverify\fR(8) server is designed to run under control
+/*	by the Postfix
 /*	master server. It maintains an optional persistent database.
 /*	To avoid being interrupted by "postfix stop" in the middle
 /*	of a database update, the process runs in a separate process
 /*	group.
 /*
-/*	This server implements the following requests:
+/*	The \fBverify\fR(8) server implements the following requests:
 /* .IP "\fBupdate\fI address status text\fR"
 /*	Update the status and text of the specified address.
 /* .IP "\fBquery\fI address\fR"
@@ -58,12 +59,13 @@
 /* CONFIGURATION PARAMETERS
 /* .ad
 /* .fi
-/*	Changes to \fBmain.cf\fR are not picked up automatically, as verify(8)
+/*	Changes to \fBmain.cf\fR are not picked up automatically,
+/*	as \fBverify\fR(8)
 /*	processes are persistent. Use the command "\fBpostfix reload\fR" after
 /*	a configuration change.
 /*
 /*	The text below provides only a parameter summary. See
-/*	postconf(5) for more details including examples.
+/*	\fBpostconf\fR(5) for more details including examples.
 /* CACHE CONTROLS
 /* .ad
 /* .fi
@@ -264,11 +266,18 @@ static int verify_parse_entry(char *buf, int *status, long *probed,
 	*probed = atol(probed_text);
 	*updated = atol(updated_text);
 	*status = atoi(buf);
+
+	/*
+	 * Coverity 200604: the code incorrectly tested (probed || updated),
+	 * so that the sanity check never detected all-zero time stamps. Such
+	 * records are never written. If we read a record with all-zero time
+	 * stamps, then something is badly broken.
+	 */
 	if ((*status == DEL_RCPT_STAT_OK
 	     || *status == DEL_RCPT_STAT_DEFER
 	     || *status == DEL_RCPT_STAT_BOUNCE
 	     || *status == DEL_RCPT_STAT_TODO)
-	    && (probed || updated))
+	    && (*probed || *updated))
 	    return (0);
     }
     msg_warn("bad address verify table entry: %.100s", buf);
@@ -303,7 +312,7 @@ static void verify_update_service(VSTREAM *client_stream)
 
     if (attr_scan(client_stream, ATTR_FLAG_STRICT,
 		  ATTR_TYPE_STR, MAIL_ATTR_ADDR, addr,
-		  ATTR_TYPE_NUM, MAIL_ATTR_ADDR_STATUS, &addr_status,
+		  ATTR_TYPE_INT, MAIL_ATTR_ADDR_STATUS, &addr_status,
 		  ATTR_TYPE_STR, MAIL_ATTR_WHY, text,
 		  ATTR_TYPE_END) == 3) {
 	/* FIX 200501 IPv6 patch did not neuter ":" in address literals. */
@@ -312,7 +321,7 @@ static void verify_update_service(VSTREAM *client_stream)
 	    msg_warn("bad recipient status %d for recipient %s",
 		     addr_status, STR(addr));
 	    attr_print(client_stream, ATTR_FLAG_NONE,
-		       ATTR_TYPE_NUM, MAIL_ATTR_STATUS, VRFY_STAT_BAD,
+		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, VRFY_STAT_BAD,
 		       ATTR_TYPE_END);
 	} else {
 
@@ -334,7 +343,7 @@ static void verify_update_service(VSTREAM *client_stream)
 		dict_put(verify_map, STR(addr), STR(buf));
 	    }
 	    attr_print(client_stream, ATTR_FLAG_NONE,
-		       ATTR_TYPE_NUM, MAIL_ATTR_STATUS, VRFY_STAT_OK,
+		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, VRFY_STAT_OK,
 		       ATTR_TYPE_END);
 	}
     }
@@ -368,7 +377,6 @@ static void verify_query_service(VSTREAM *client_stream)
     long    probed;
     long    updated;
     char   *text;
-    VSTREAM *post;
 
     if (attr_scan(client_stream, ATTR_FLAG_STRICT,
 		  ATTR_TYPE_STR, MAIL_ATTR_ADDR, addr,
@@ -422,8 +430,8 @@ static void verify_query_service(VSTREAM *client_stream)
 	 * Respond to the client.
 	 */
 	attr_print(client_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_NUM, MAIL_ATTR_STATUS, VRFY_STAT_OK,
-		   ATTR_TYPE_NUM, MAIL_ATTR_ADDR_STATUS, addr_status,
+		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, VRFY_STAT_OK,
+		   ATTR_TYPE_INT, MAIL_ATTR_ADDR_STATUS, addr_status,
 		   ATTR_TYPE_STR, MAIL_ATTR_WHY, text,
 		   ATTR_TYPE_END);
 
@@ -451,8 +459,9 @@ static void verify_query_service(VSTREAM *client_stream)
 			 STR(addr), addr_status, now, updated);
 	    post_mail_fopen_async(strcmp(var_verify_sender, "<>") == 0 ?
 				  "" : var_verify_sender, STR(addr),
-				  CLEANUP_FLAG_MASK_INTERNAL,
-				  DEL_REQ_FLAG_VERIFY,
+				  INT_FILT_NONE,
+				  DEL_REQ_FLAG_MTA_VRFY,
+				  (VSTRING *) 0,
 				  verify_post_mail_action,
 				  (void *) 0);
 	    if (updated != 0 || var_verify_neg_cache != 0) {
@@ -501,7 +510,7 @@ static void verify_service(VSTREAM *client_stream, char *unused_service,
 	} else {
 	    msg_warn("unrecognized request: \"%s\", ignored", STR(request));
 	    attr_print(client_stream, ATTR_FLAG_NONE,
-		       ATTR_TYPE_NUM, MAIL_ATTR_STATUS, VRFY_STAT_BAD,
+		       ATTR_TYPE_INT, MAIL_ATTR_STATUS, VRFY_STAT_BAD,
 		       ATTR_TYPE_END);
 	}
     }
